package cn.estsh.i3plus.ext.mes.apiservice.controller.board;

import cn.estsh.i3plus.ext.mes.api.busi.board.IMesYfBoardService;
import cn.estsh.i3plus.ext.mes.pojo.constant.MesCommonConstant;
import cn.estsh.i3plus.ext.mes.pojo.model.board.MesWorkCenterBoardConditionModel;
import cn.estsh.i3plus.pojo.base.enumutil.ResourceEnumUtil;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.exception.ImppBusiException;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ResultBean;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * @Description : 产线看板
 * @Reference :
 * @Author : logic
 * @CreateDate : 2024/6/19 11:10
 * @Modify:
 **/
@Api("产线看板")
@RestController
@RequestMapping(MesCommonConstant.MES_YANFEN + "/work-center-board")
public class MesWorkCenterBoardController {

    @Autowired
    private IMesYfBoardService mesYfBoardService;

    @ApiOperation(value = "查询产线状态", notes = "查询产线状态")
    @PostMapping("/query-work-center-status")
    public ResultBean queryWorkCenterStatus(@RequestBody MesWorkCenterBoardConditionModel conditionModel){
        try {
            ValidatorBean.beginValid(conditionModel)
                    .notNull("workCenterCodeList", conditionModel.getWorkCenterCodeList())
                    .notNull("timeRangeIdentification", conditionModel.getTimeRangeIdentification());
            String organizeCode = AuthUtil.getOrganize().getOrganizeCode();
            conditionModel.setOrganizeCode(organizeCode);
            return ResultBean.success("查询产线状态").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode()).
                    setResultList(mesYfBoardService.queryWorkCenterStatus(conditionModel));
        } catch (ImppBusiException e) {
            return ResultBean.fail(e);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @ApiOperation(value = "查询产线生产完成率", notes = "查询产线生产完成率")
    @PostMapping("/query-work-center-production-completion-rate")
    public ResultBean queryWorkCenterProductionCompletionRate(@RequestBody MesWorkCenterBoardConditionModel conditionModel){
        try {
            ValidatorBean.beginValid(conditionModel)
                    .notNull("workCenterCodeList", conditionModel.getWorkCenterCodeList())
                    .notNull("timeRangeIdentification", conditionModel.getTimeRangeIdentification());
            String organizeCode = AuthUtil.getOrganize().getOrganizeCode();
            conditionModel.setOrganizeCode(organizeCode);
            return ResultBean.success("查询产线生产完成率").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode()).
                    setResultObject(mesYfBoardService.queryWorkCenterProductionCompletionRate(conditionModel));
        } catch (ImppBusiException e) {
            return ResultBean.fail(e);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @ApiOperation(value = "查询产线工单完成情况", notes = "查询产线工单完成情况")
    @PostMapping("/query-work-center-work-order-completion-status")
    public ResultBean queryWorkCenterWorkOrderCompletionStatus(@RequestBody MesWorkCenterBoardConditionModel conditionModel){
        try {
            ValidatorBean.beginValid(conditionModel)
                    .notNull("workCenterCodeList", conditionModel.getWorkCenterCodeList())
                    .notNull("timeRangeIdentification", conditionModel.getTimeRangeIdentification());
            String organizeCode = AuthUtil.getOrganize().getOrganizeCode();
            conditionModel.setOrganizeCode(organizeCode);
            return ResultBean.success("查询产线工单完成情况").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode()).
                    setResultList(mesYfBoardService.queryWorkCenterWorkOrderCompletionStatus(conditionModel));
        } catch (ImppBusiException e) {
            return ResultBean.fail(e);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @ApiOperation(value = "查询单位小时完工数", notes = "查询单位小时完工数")
    @PostMapping("/query-unit-hour-completion-rate")
    public ResultBean queryUnitHourCompletionRate(@RequestBody MesWorkCenterBoardConditionModel conditionModel){
        try {
            ValidatorBean.beginValid(conditionModel)
                    .notNull("workCenterCodeList", conditionModel.getWorkCenterCodeList())
                    .notNull("timeRangeIdentification", conditionModel.getTimeRangeIdentification());
            String organizeCode = AuthUtil.getOrganize().getOrganizeCode();
            conditionModel.setOrganizeCode(organizeCode);
            return ResultBean.success("查询单位小时完工数").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode()).
                    setResultList(mesYfBoardService.queryUnitHourCompletionRate(conditionModel));
        } catch (ImppBusiException e) {
            return ResultBean.fail(e);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @ApiOperation(value = "查询产线一次下线合格率", notes = "查询产线一次下线合格率")
    @PostMapping("/query-work-center-first-time-offline-qualification-rate")
    public ResultBean queryWorkCenterFirstTimeOfflineQualificationRate(@RequestBody MesWorkCenterBoardConditionModel conditionModel){
        try {
            ValidatorBean.beginValid(conditionModel)
                    .notNull("workCenterCodeList", conditionModel.getWorkCenterCodeList())
                    .notNull("timeRangeIdentification", conditionModel.getTimeRangeIdentification());
            String organizeCode = AuthUtil.getOrganize().getOrganizeCode();
            conditionModel.setOrganizeCode(organizeCode);
            return ResultBean.success("查询产线一次下线合格率").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode()).
                    setResultList(mesYfBoardService.queryWorkCenterFirstTimeOfflineQualificationRate(conditionModel));
        } catch (ImppBusiException e) {
            return ResultBean.fail(e);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @ApiOperation(value = "查询点检结果", notes = "查询点检结果")
    @PostMapping("/query-inspection-results")
    public ResultBean queryInspectionResults(@RequestBody MesWorkCenterBoardConditionModel conditionModel){
        try {
            ValidatorBean.beginValid(conditionModel)
                    .notNull("workCenterCodeList", conditionModel.getWorkCenterCodeList())
                    .notNull("timeRangeIdentification", conditionModel.getTimeRangeIdentification());
            String organizeCode = AuthUtil.getOrganize().getOrganizeCode();
            conditionModel.setOrganizeCode(organizeCode);
            return ResultBean.success("查询点检结果").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode()).
                    setResultList(mesYfBoardService.queryInspectionResults(conditionModel));
        } catch (ImppBusiException e) {
            return ResultBean.fail(e);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @ApiOperation(value = "查询异常停线时间", notes = "查询异常停线时间")
    @PostMapping("/query-abnormal-downtime")
    public ResultBean queryAbnormalDowntime(@RequestBody MesWorkCenterBoardConditionModel conditionModel){
        try {
            ValidatorBean.beginValid(conditionModel)
                    .notNull("workCenterCodeList", conditionModel.getWorkCenterCodeList())
                    .notNull("timeRangeIdentification", conditionModel.getTimeRangeIdentification());
            String organizeCode = AuthUtil.getOrganize().getOrganizeCode();
            conditionModel.setOrganizeCode(organizeCode);
            return ResultBean.success("查询异常停线时间").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode()).
                    setResultObject(mesYfBoardService.queryAbnormalDowntime(conditionModel));
        } catch (ImppBusiException e) {
            return ResultBean.fail(e);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

}
