package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base;

import cn.estsh.i3plus.ext.mes.api.base.IMesEquipmentSpotCheckPartService;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.base.util.StringUtil;
import cn.estsh.i3plus.pojo.mes.bean.MesEquipmentSpotCheck;
import cn.estsh.i3plus.pojo.mes.bean.MesEquipmentSpotCheckPart;
import cn.estsh.i3plus.pojo.mes.repository.MesEquipmentSpotCheckRepository;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

@Service
@Slf4j
public class MesEquipmentSpotCheckPartService extends BaseMesService<MesEquipmentSpotCheckPart> implements IMesEquipmentSpotCheckPartService {


    @Autowired
    private MesEquipmentSpotCheckRepository equipmentSpotCheckRepository;
    protected void setPackQueryBean(MesEquipmentSpotCheckPart bean, DdlPackBean packBean) {
        DdlPreparedPack.getNumEqualPack(bean.getPid(), "pid", packBean);
        DdlPreparedPack.getStringLikerPack(bean.getPartNo(), "partNo", packBean);
    }
    @Override
    public void validateImport(List<MesEquipmentSpotCheckPart> beanList) {

        for (MesEquipmentSpotCheckPart item : beanList) {
            // 数据校验
            if(StringUtil.isEmpty(item.getPid())){
                ValidatorBean.checkNotNull(item.getSpotCheckCode(), "点检项目代码不能为空");

                DdlPackBean seriesPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
                DdlPreparedPack.getStringEqualPack(item.getSpotCheckCode(), "spotCheckCode", seriesPackBean);
                MesEquipmentSpotCheckPart itemFlg = baseRDao.getByProperty(seriesPackBean);
                if (StringUtil.isEmpty(itemFlg)) {
                    throw ImppExceptionBuilder.newInstance()
                            .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                            .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                            .setErrorDetail("【%s】点检项目代码不存在，请检查数据", item.getSpotCheckCode())
                            .build();
                }
                item.setPid(itemFlg.getPid());
            }else {
                ValidatorBean.checkNotNull(item.getId(), "点检项目id不能为空");

                DdlPackBean seriesPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
                DdlPreparedPack.getNumNOEqualPack(item.getId(), "id", seriesPackBean);

                if (!baseRDao.isExitByHql(seriesPackBean)) {
                    throw ImppExceptionBuilder.newInstance()
                            .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                            .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                            .setErrorDetail("【%s】点检项目id不存在，请检查数据", item.getPid())
                            .build();
                }
            }
        }
    }
    @Override
    public List<MesEquipmentSpotCheckPart> validateReturnImport(List<MesEquipmentSpotCheckPart> beanList) {

        for (MesEquipmentSpotCheckPart item : beanList) {
            // 数据校验
            if(StringUtil.isEmpty(item.getPid())){
                ValidatorBean.checkNotNull(item.getSpotCheckCode(), "点检项目代码不能为空");

                DdlPackBean seriesPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
                DdlPreparedPack.getStringEqualPack(item.getSpotCheckCode(), "spotCheckCode", seriesPackBean);
                MesEquipmentSpotCheck itemFlg = equipmentSpotCheckRepository.getByProperty(seriesPackBean);
                if (StringUtil.isEmpty(itemFlg)) {
                    throw ImppExceptionBuilder.newInstance()
                            .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                            .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                            .setErrorDetail("【%s】点检项目代码不存在，请检查数据", item.getSpotCheckCode())
                            .build();
                }
                item.setPid(itemFlg.getId());
            }else {
                ValidatorBean.checkNotNull(item.getId(), "点检项目id不能为空");

                DdlPackBean seriesPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
                DdlPreparedPack.getNumNOEqualPack(item.getId(), "id", seriesPackBean);

                if (!baseRDao.isExitByHql(seriesPackBean)) {
                    throw ImppExceptionBuilder.newInstance()
                            .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                            .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                            .setErrorDetail("【%s】点检项目id不存在，请检查数据", item.getPid())
                            .build();
                }
            }
        }
        return beanList;
    }
}
