package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base;

import cn.estsh.i3plus.ext.mes.api.base.IMesPackingDefineDetailsService;
import cn.estsh.i3plus.ext.mes.apiservice.utils.MesException;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.MesPackingDefineDetails;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

/**
 * @Description : 包装定义维护明细
 * @Reference :
 * @Author : junsheng.li
 * @CreateDate 2024/6/3 14:41
 * @Modify:
 **/
@Service
@Slf4j
public class MesPackingDefineDetailsServiceImpl extends BaseMesService<MesPackingDefineDetails> implements IMesPackingDefineDetailsService {

    @Override
    protected void setPackQueryBean(MesPackingDefineDetails bean, DdlPackBean packBean) {
        DdlPreparedPack.getStringEqualPack(bean.getPackCode(), "packCode", packBean);
        DdlPreparedPack.getStringLikerPack(bean.getPartNo(), "partNo", packBean);
        DdlPreparedPack.getStringLikerPack(bean.getPartName(), "partName", packBean);
    }

    @Override
    protected void onInsertBean(MesPackingDefineDetails item) {
        // 数据校验
        ValidatorBean.checkNotNull(item.getPackCode(), "箱类别代号不能为空");
        ValidatorBean.checkNotNull(item.getPartNo(), "零件号不能为空");

        DdlPackBean roadPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(item.getPackCode(), "packCode", roadPackBean);
        DdlPreparedPack.getStringEqualPack(item.getPartNo(), "partNo", roadPackBean);

        boolean flg = baseRDao.isExitByHql(roadPackBean);
        if (flg) {
            MesException.throwMesBusiException("箱类别代号【%s】物料号【%s】已经存在，请检查数据", item.getPackCode(),item.getPartNo());
        }
    }

    @Override
    protected void onUpdateBean(MesPackingDefineDetails item) {
        // 数据校验
        ValidatorBean.checkNotNull(item.getPackCode(), "箱类别代号不能为空");
        ValidatorBean.checkNotNull(item.getPartNo(), "零件号不能为空");


        DdlPackBean roadPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(item.getPackCode(), "packCode", roadPackBean);
        DdlPreparedPack.getStringEqualPack(item.getPartNo(), "partNo", roadPackBean);
        DdlPreparedPack.getNumNOEqualPack(item.getId(), "id", roadPackBean);
        boolean flg = baseRDao.isExitByHql(roadPackBean);
        if (flg) {
            MesException.throwMesBusiException("箱类别代号【%s】物料号【%s】已经存在，请检查数据", item.getPackCode(),item.getPartNo());
        }
    }
}
