package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base;

import cn.estsh.i3plus.ext.mes.api.base.IMesPartTransferDetailService;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.base.util.StringUtil;
import cn.estsh.i3plus.pojo.mes.bean.MesEquipmentSpotCheck;
import cn.estsh.i3plus.pojo.mes.bean.MesEquipmentSpotCheckPart;
import cn.estsh.i3plus.pojo.mes.bean.MesPartTransfer;
import cn.estsh.i3plus.pojo.mes.bean.MesPartTransferDetail;
import cn.estsh.i3plus.pojo.mes.repository.MesPartTransferDetailRepository;
import cn.estsh.i3plus.pojo.mes.repository.MesPartTransferRepository;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

@Service
@Slf4j
public class MesPartTransferDetailService extends BaseMesService<MesPartTransferDetail> implements IMesPartTransferDetailService {

    @Autowired
    private MesPartTransferRepository partTransferRepository;

    protected void setPackQueryBean(MesPartTransferDetail bean, DdlPackBean packBean) {
        DdlPreparedPack.getNumEqualPack(bean.getPid(),"pid",packBean);
        DdlPreparedPack.getStringEqualPack(bean.getCustCode(),"custCode",packBean);
        DdlPreparedPack.getStringEqualPack(bean.getCustPartNo(),"custPartNo",packBean);
        DdlPreparedPack.getStringEqualPack(bean.getCustOrganizeCode(),"custOrganizeCode",packBean);
        DdlPreparedPack.getStringEqualPack(bean.getFeatureNo(),"featureNo",packBean);
        DdlPreparedPack.getStringEqualPack(bean.getMessageNo(),"messageNo",packBean);
        DdlPreparedPack.getStringEqualPack(bean.getPartNo(),"partNo",packBean);
    }
    @Override
    protected void onInsertBean(MesPartTransferDetail item) {
        // 数据校验
        ValidatorBean.checkNotNull(item.getOrganizeCode(), "工厂号不能为空");
        ValidatorBean.checkNotNull(item.getPartNo(), "partNo不能为空");
        ValidatorBean.checkNotNull(item.getCustPartNo(), "custPartNo不能为空" );
        ValidatorBean.checkNotNull(item.getFeatureNo(), "featureNo不能为空");
//        ValidatorBean.checkNotNull(item.getCustCode(), "custCode不能为空" );+
//        ValidatorBean.checkNotNull(item.getMessageNo(), "messageNo不能为空");
//        ValidatorBean.checkNotNull(item.getCustOrganizeCode(), "custOrganizeCode不能为空" );

        //唯一性校验
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(item.getPartNo(),"partNo",packBean);
        DdlPreparedPack.getStringEqualPack(item.getCustPartNo(),"custPartNo",packBean);
        DdlPreparedPack.getStringEqualPack(item.getFeatureNo(),"featureNo",packBean);
//        DdlPreparedPack.getStringEqualPack(item.getCustOrganizeCode(),"custOrganizeCode",packBean);
//        DdlPreparedPack.getStringEqualPack(item.getMessageNo(),"messageNo",packBean);
//        DdlPreparedPack.getStringEqualPack(item.getCustCode(),"custCode",packBean);

        boolean flg = baseRDao.isExitByHql(packBean);
        if (flg) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("客户零件转化明细客户零件号【%s】零件号【%s】特征值【%s】已经存在，请检查数据",
                            item.getCustPartNo(),item.getPartNo(), item.getMessageNo())
                    .build();
        }
    }
    @Override
    protected void onUpdateBean(MesPartTransferDetail item) {
        // 数据校验
        ValidatorBean.checkNotNull(item.getOrganizeCode(), "工厂号不能为空");
        ValidatorBean.checkNotNull(item.getPartNo(), "partNo不能为空");
        ValidatorBean.checkNotNull(item.getCustPartNo(), "custPartNo不能为空" );
        ValidatorBean.checkNotNull(item.getFeatureNo(), "featureNo不能为空");


        //唯一性校验
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(item.getPartNo(),"partNo",packBean);
        DdlPreparedPack.getStringEqualPack(item.getCustPartNo(),"custPartNo",packBean);
        DdlPreparedPack.getStringEqualPack(item.getFeatureNo(),"featureNo",packBean);
        DdlPreparedPack.getNumNOEqualPack(item.getId(), "id", packBean);

        boolean flg = baseRDao.isExitByHql(packBean);
        if (flg) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("客户零件转化明细客户零件号【%s】零件号【%s】特征值【%s】已经存在，请检查数据",
                            item.getCustPartNo(),item.getPartNo(), item.getMessageNo())
                    .build();
        }
    }

    @Override
    public List<MesPartTransferDetail> validateReturnImport(List<MesPartTransferDetail> beanList) {

        for (MesPartTransferDetail item : beanList) {
            // 数据校验
            if(StringUtil.isEmpty(item.getPid())){
                ValidatorBean.checkNotNull(item.getCustCode(), "客户代码不能为空");
                ValidatorBean.checkNotNull(item.getCustOrganizeCode(), "客户工厂代码不能为空");
                ValidatorBean.checkNotNull(item.getMessageNo(), "信息点代码不能为空");



                DdlPackBean seriesPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
                DdlPreparedPack.getStringEqualPack(item.getCustCode(), "custCode", seriesPackBean);
                DdlPreparedPack.getStringEqualPack(item.getCustOrganizeCode(), "custOrganizeCode", seriesPackBean);
                DdlPreparedPack.getStringEqualPack(item.getMessageNo(), "messageNo", seriesPackBean);


                MesPartTransfer itemFlg = partTransferRepository.getByProperty(seriesPackBean);
                if (StringUtil.isEmpty(itemFlg)) {
                    throw ImppExceptionBuilder.newInstance()
                            .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                            .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                            .setErrorDetail("【%s】客户代码，【%s】客户工厂代码,【%s】信息点代码请检查数据", item.getCustCode(),item.getCustOrganizeCode(), item.getMessageNo())
                            .build();
                }
                item.setPid(itemFlg.getId());
            }
        }
        return beanList;
    }
}
