package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base;

import cn.estsh.i3plus.ext.mes.api.base.IBaseMesService;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.MesPartTypeDefect;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.util.List;

@Service
@Slf4j
public class MesPartTypeDefectService extends BaseMesService<MesPartTypeDefect> implements IBaseMesService<MesPartTypeDefect> {


    @Override
    protected void setPackQueryBean(MesPartTypeDefect bean, DdlPackBean packBean) {
        DdlPreparedPack.getStringEqualPack(bean.getPartTypeCode(), "partTypeCode", packBean);
        DdlPreparedPack.getStringEqualPack(bean.getDefectTypeCode(), "defectTypeCode", packBean);
    }

    @Override
    protected void onInsertBean(MesPartTypeDefect item) {

        // 数据校验
        ValidatorBean.checkNotNull(item.getDefectTypeCode(), "缺陷类型不能为空");
        ValidatorBean.checkNotNull(item.getPartTypeCode(), "零件类型不能为空");

        DdlPackBean roadPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(item.getDefectTypeCode(), "defectTypeCode", roadPackBean);
        DdlPreparedPack.getStringEqualPack(item.getPartTypeCode(), "partTypeCode", roadPackBean);

        boolean flg = baseRDao.isExitByHql(roadPackBean);
        if (flg) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("缺陷类型【%s】零件类型【%s】已经存在，请检查数据", item.getDefectTypeCode(), item.getPartTypeCode())
                    .build();
        }

    }

    @Override
    protected void onUpdateBean(MesPartTypeDefect item) {
        // 数据校验
        ValidatorBean.checkNotNull(item.getDefectTypeCode(), "缺陷类型不能为空");
        ValidatorBean.checkNotNull(item.getPartTypeCode(), "零件类型不能为空");

        DdlPackBean roadPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(item.getDefectTypeCode(), "defectTypeCode", roadPackBean);
        DdlPreparedPack.getStringEqualPack(item.getPartTypeCode(), "partTypeCode", roadPackBean);
        DdlPreparedPack.getNumNOEqualPack(item.getId(), "id", roadPackBean);
        boolean flg = baseRDao.isExitByHql(roadPackBean);
        if (flg) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("缺陷类型【%s】零件类型【%s】已经存在，请检查数据", item.getDefectTypeCode(), item.getPartTypeCode())
                    .build();
        }
    }
    @Override
    public void validateImport(List<MesPartTypeDefect> beanList) {

        for (MesPartTypeDefect item : beanList) {
            // 数据校验
            ValidatorBean.checkNotNull(item.getDefectTypeCode(), "缺陷类型不能为空");
            ValidatorBean.checkNotNull(item.getPartTypeCode(), "零件类型不能为空");

            DdlPackBean roadPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
            DdlPreparedPack.getStringEqualPack(item.getDefectTypeCode(), "defectTypeCode", roadPackBean);
            DdlPreparedPack.getStringEqualPack(item.getPartTypeCode(), "partTypeCode", roadPackBean);

            boolean flg = baseRDao.isExitByHql(roadPackBean);
            if (flg) {
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                        .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                        .setErrorDetail("缺陷类型【%s】零件类型【%s】已经存在，请检查数据", item.getDefectTypeCode(), item.getPartTypeCode())
                        .build();
            }
        }
    }
}
