package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base;

import cn.estsh.i3plus.ext.mes.api.base.IMesCraftRouteService;
import cn.estsh.i3plus.ext.mes.api.base.IMesPartService;
import cn.estsh.i3plus.ext.mes.api.base.IMesProdCraftRouteService;
import cn.estsh.i3plus.ext.mes.apiservice.utils.MesException;
import cn.estsh.i3plus.platform.common.util.MesConstWords;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.base.util.StringUtil;
import cn.estsh.i3plus.pojo.mes.bean.MesCraftRoute;
import cn.estsh.i3plus.pojo.mes.bean.MesPart;
import cn.estsh.i3plus.pojo.mes.bean.MesProdCraftRoute;
import cn.estsh.i3plus.pojo.mes.bean.MesWorkCenter;
import cn.estsh.i3plus.pojo.mes.repository.MesProdRouteCfgRepository;
import cn.estsh.i3plus.pojo.mes.repository.MesWorkCenterRepository;
import cn.estsh.i3plus.pojo.mes.util.MesExtEnumUtil;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.List;
import java.util.Objects;

/**
 * @Description : 产品工艺路线
 * @Reference :
 * @Author : gsz
 * @CreateDate 2024/5/13 13:13
 * @Modify:
 **/
@Service
@Slf4j
public class MesProdCraftRouteServiceImpl extends BaseMesService<MesProdCraftRoute> implements IMesProdCraftRouteService {

    @Autowired
    private IMesCraftRouteService mesCraftRouteService;
    @Autowired
    private MesWorkCenterRepository mesWorkCenterRDao;
    @Autowired
    private IMesPartService mesPartService;
    @Autowired
    private MesProdRouteCfgRepository mesProdRouteCfgRepository;

    @Override
    protected void setPackQueryBean(MesProdCraftRoute bean, DdlPackBean packBean) {
        DdlPreparedPack.getStringEqualPack(bean.getCraftRouteCode(), "craftRouteCode", packBean);
//        DdlPreparedPack.getStringLikerPack(bean.getCraftRouteName(), "craftRouteName", packBean);
        DdlPreparedPack.getNumEqualPack(bean.getCraftRouteType(), "craftRouteType", packBean);
        DdlPreparedPack.getStringEqualPack(bean.getWorkCenterCode(), "workCenterCode", packBean);
        DdlPreparedPack.getStringEqualPack(bean.getPptCode(), "pptCode", packBean);
        DdlPreparedPack.getStringLikerPack(bean.getPartNo(), "partNo", packBean);
    }

    @Override
    protected void onInsertBean(MesProdCraftRoute item) {
        // 数据校验
        checkData(item);
    }


    @Override
    protected void onUpdateBean(MesProdCraftRoute item) {
        // 数据校验
        checkData(item);
    }

    private void checkData(MesProdCraftRoute item) {
        // 数据校验
        ValidatorBean.checkNotNull(item.getOrganizeCode(), "工厂号不能为空");
        ValidatorBean.checkNotNull(item.getCraftRouteCode(), "工艺路线代码不能为空");
        ValidatorBean.checkNotNull(item.getCraftRouteName(), "工艺路线名称不能为空");
        ValidatorBean.checkNotNull(item.getCraftRouteType(), "工艺路线类型不能为空");

        String partNo = item.getPartNo();
        String pptCode = item.getPptCode();
        String workCenterCode = item.getWorkCenterCode();
        //校验产线
        MesCraftRoute mesCraftRoute = mesCraftRouteService.getMesCraftRouteByCode(item.getCraftRouteCode(), item.getOrganizeCode());
        if (!StringUtil.isEmpty(workCenterCode)) {//生产线维度配置  工艺路线类型必须与产线类型保持一致
            //校验产线
            MesWorkCenter mesWorkCenter = mesWorkCenterRDao.getByProperty(
                    new String[]{MesConstWords.ORGANIZE_CODE, MesConstWords.IS_DELETED, MesConstWords.IS_VALID, "workCenterCode"},
                    new Object[]{item.getOrganizeCode(), CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue(),
                            CommonEnumUtil.IS_VAILD.VAILD.getValue(), item.getWorkCenterCode()});
            if (null == mesWorkCenter) {
                MesException.throwMesBusiException("产线代码【%s】无效", item.getWorkCenterCode());
            }
            if (!Objects.equals(mesCraftRoute.getCraftRouteType(), mesWorkCenter.getCenterType())) {
                MesException.throwMesBusiException("工艺路线代码【%s】工艺路线类型必须与产线类型保持一致，请检查数据", item.getCraftRouteCode());
            }
            item.setAreaCode(mesWorkCenter.getAreaCode());
        }
        //零件号&产型只能维护一个
        if (!StringUtil.isEmpty(partNo) && !StringUtil.isEmpty(pptCode)) {
            MesException.throwMesBusiException("工艺路线代码【%s】【%s】，零件与零件产品类型二选一维护，请检查数据", item.getCraftRouteCode(),MesExtEnumUtil.WORK_CENTER_TYPE.valueOfDescription(item.getCraftRouteType()));
        }
        //20 非排序类型
        if (MesExtEnumUtil.WORK_CENTER_TYPE.NOSORT.getValue() == item.getCraftRouteType()) {
            if(StringUtil.isEmpty(workCenterCode) && StringUtil.isEmpty(partNo) && StringUtil.isEmpty(pptCode)){
                MesException.throwMesBusiException("工艺路线代码【%s】【%s】，请选择产线、零件、零件产品类，请检查数据", item.getCraftRouteCode(),MesExtEnumUtil.WORK_CENTER_TYPE.valueOfDescription(item.getCraftRouteType()));
            }
        } else {//10 排序类型
            if (StringUtil.isEmpty(workCenterCode)) {
                MesException.throwMesBusiException("工艺路线代码【%s】【%s】，产线代码不能为空，请检查数据", item.getCraftRouteCode(), MesExtEnumUtil.WORK_CENTER_TYPE.valueOfDescription(item.getCraftRouteType()));
            }
            //校验产线是否绑定有流程
            DdlPackBean packBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
            DdlPreparedPack.getStringEqualPack(workCenterCode,"workCenterCode",packBean);
            if(!mesProdRouteCfgRepository.isExitByHql(packBean)){
                MesException.throwMesBusiException("【%s】【%s】未绑定流程信息，请检查数据",  MesExtEnumUtil.WORK_CENTER_TYPE.valueOfDescription(item.getCraftRouteType()),workCenterCode);
            }
        }
        //唯一性校验
        checkExist(item);
    }

    private void checkExist(MesProdCraftRoute item) {
        DdlPackBean roadPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
        DdlPreparedPack.getNumEqualPack(item.getCraftRouteType(), "craftRouteType", roadPackBean);
        if(StringUtil.isEmpty(item.getWorkCenterCode())){
            DdlPreparedPack.getIsNull("workCenterCode", roadPackBean);
        }else{
            DdlPreparedPack.getStringEqualPack(item.getWorkCenterCode(), "workCenterCode", roadPackBean);
        }
        if(StringUtils.isEmpty(item.getPartNo())){
            DdlPreparedPack.getIsNull("partNo", roadPackBean);
        }else{
            DdlPreparedPack.getStringEqualPack(item.getPartNo(), "partNo", roadPackBean);
        }
        if(StringUtils.isEmpty(item.getPptCode())){
            DdlPreparedPack.getIsNull("pptCode", roadPackBean);
        }else{
            DdlPreparedPack.getStringEqualPack(item.getPptCode(), "pptCode", roadPackBean);
        }
        DdlPreparedPack.getNumNOEqualPack(item.getId(), "id", roadPackBean);
        if (baseRDao.isExitByHql(roadPackBean)) {
            MesException.throwMesBusiException("工艺路线类型【%s】生产线代码【%s】物料编码【%s】物料产品类型【%s】数据已经存在，请检查数据",
                    MesExtEnumUtil.WORK_CENTER_TYPE.valueOfDescription(item.getCraftRouteType()), item.getWorkCenterCode(), item.getPartNo(), item.getPptCode());
        }
    }

    @Override
    public String getCraftRouteCode(String organizeCode, String workCenterCode, String partNo) {
        //按照物料查找工艺路线
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getStringEqualPack(partNo, MesConstWords.PART_NO, packBean);
        DdlPreparedPack.getIsNull("pptCode", packBean);
        List<MesProdCraftRoute> prodCraftRouteList = baseRDao.findByHqlWhere(packBean);
        if (!CollectionUtils.isEmpty(prodCraftRouteList)) {
            return prodCraftRouteList.iterator().next().getCraftRouteCode();
        }

        MesPart partDb = mesPartService.getPartByPartNo(partNo, organizeCode);
        if (null == partDb || StringUtils.isEmpty(partDb.getPptCode())) {
            return null;
        }

        //按照物料产品类型查找工艺路线
        packBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getStringEqualPack(partDb.getPptCode(), "pptCode", packBean);
        DdlPreparedPack.getIsNull(MesConstWords.PART_NO, packBean);
        prodCraftRouteList = baseRDao.findByHqlWhere(packBean);
        if (!CollectionUtils.isEmpty(prodCraftRouteList)) {
            return prodCraftRouteList.iterator().next().getCraftRouteCode();
        }

        //按照产线查找工艺路线
        packBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getStringEqualPack(workCenterCode, MesConstWords.WORK_CENTER_CODE, packBean);
        DdlPreparedPack.getIsNull(MesConstWords.PART_NO, packBean);
        DdlPreparedPack.getIsNull("pptCode", packBean);
        prodCraftRouteList = baseRDao.findByHqlWhere(packBean);
        if (!CollectionUtils.isEmpty(prodCraftRouteList)) {
            return prodCraftRouteList.iterator().next().getCraftRouteCode();
        }
        return null;
    }
}
