package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base;

import cn.estsh.i3plus.ext.mes.api.base.IMesPartSapService;
import cn.estsh.i3plus.ext.mes.api.base.IMesProdRuleNosortCfgService;
import cn.estsh.i3plus.ext.mes.apiservice.dao.IMesProdRuleNosortCfgDao;
import cn.estsh.i3plus.ext.mes.apiservice.utils.MesException;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.MesPartSap;
import cn.estsh.i3plus.pojo.mes.bean.MesProdRuleNosortCfg;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.List;

@Service
@Slf4j
public class MesProdRuleNosortCfgService extends BaseMesService<MesProdRuleNosortCfg> implements IMesProdRuleNosortCfgService {

    @Autowired
    private IMesProdRuleNosortCfgDao mesProdRuleNosortCfgDao;

    @Autowired
    private IMesPartSapService partSapService;

    protected void setPackQueryBean(MesProdRuleNosortCfg bean, DdlPackBean packBean) {
        DdlPreparedPack.getStringLikerPack(bean.getInPartNo(), "inPartNo", packBean);
        DdlPreparedPack.getStringLikerPack(bean.getOutPartNo(), "outPartNo", packBean);
        DdlPreparedPack.getStringEqualPack(bean.getEquipmentCode(), "equipmentCode", packBean);

    }

    @Override
    public MesProdRuleNosortCfg insert(MesProdRuleNosortCfg bean) {
        //插入前构造并校验数据
        onInsertBean(bean);

        bean = baseRDao.insert(bean);
        return bean;
    }

    protected void onInsertBean(MesProdRuleNosortCfg item) {
        // 数据校验
        //ValidatorBean.checkNotNull(item.getInPartNo(), "进件零件号不能为空");
        ValidatorBean.checkNotNull(item.getOutPartNo(), "产成零件号不能为空");
        ValidatorBean.checkNotNull(item.getOutPartNo(), "设备不能为空");

        DdlPackBean seriesPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
        if (StringUtils.isEmpty(item.getInPartNo())) {
            DdlPreparedPack.getStringEqualPack(item.getInPartNo(), "inPartNo", seriesPackBean);
        }
        DdlPreparedPack.getStringEqualPack(item.getOutPartNo(), "outPartNo", seriesPackBean);
        DdlPreparedPack.getStringEqualPack(item.getEquipmentCode(), "equipmentCode", seriesPackBean);
        if (!StringUtils.isEmpty(item.getInPartNo())) {
            MesPartSap inPart = partSapService.getMesPartSapByPartNo(item.getInPartNo(), item.getOrganizeCode());
            if (inPart == null) {
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                        .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                        .setErrorDetail("进料零件号不存在，请检查数据")
                        .build();
            }
        }
        if (!StringUtils.isEmpty(item.getOutPartNo())) {
            MesPartSap outPart = partSapService.getMesPartSapByPartNo(item.getOutPartNo(),item.getOrganizeCode());
            if (outPart == null) {
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                        .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                        .setErrorDetail("产成零件号不存在，请检查数据")
                        .build();
            }
        }
        boolean flg = baseRDao.isExitByHql(seriesPackBean);
        if (flg) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("该非排序加工规则已存在，请检查数据")
                    .build();
        }

    }

    protected void onUpdateBean(MesProdRuleNosortCfg item) {
        ValidatorBean.checkNotNull(item.getOutPartNo(), "产成零件号不能为空");
        ValidatorBean.checkNotNull(item.getOutPartNo(), "设备不能为空");

        DdlPackBean seriesPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
        if (StringUtils.isEmpty(item.getInPartNo())) {
            DdlPreparedPack.getStringEqualPack(item.getInPartNo(), "inPartNo", seriesPackBean);
        }
        DdlPreparedPack.getStringEqualPack(item.getOutPartNo(), "outPartNo", seriesPackBean);
        DdlPreparedPack.getStringEqualPack(item.getEquipmentCode(), "equipmentCode", seriesPackBean);
        if (!StringUtils.isEmpty(item.getInPartNo())) {
            MesPartSap inPart = partSapService.getMesPartSapByPartNo(item.getInPartNo(), item.getOrganizeCode());
            if (inPart == null) {
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                        .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                        .setErrorDetail("进料零件号不存在，请检查数据")
                        .build();
            }
        }
        if (!StringUtils.isEmpty(item.getOutPartNo())) {
            MesPartSap outPart = partSapService.getMesPartSapByPartNo(item.getOutPartNo(), item.getOrganizeCode());
            if (outPart == null) {
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                        .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                        .setErrorDetail("产成零件号不存在，请检查数据")
                        .build();
            }
        }
    }

    @Override
    public MesProdRuleNosortCfg findMesProdRuleNosortCfgByWorkCenterCode(String workCenterCode, String partNo, String organizeCode) {
        List<MesProdRuleNosortCfg> prodRuleNosortCfgList = mesProdRuleNosortCfgDao.findMesProdRuleNosortCfgByWorkCenterCode(workCenterCode, partNo, organizeCode);
        if(CollectionUtils.isEmpty(prodRuleNosortCfgList)){
            MesException.throwMesBusiException("产线【%s】产出零件【%s】对应的非排序工艺路线信息不存在",workCenterCode,partNo);
        }
        return prodRuleNosortCfgList.iterator().next();
    }
    @Override
    public void validateImport(List<MesProdRuleNosortCfg> beanList) {

        for (MesProdRuleNosortCfg item : beanList) {
            // 数据校验
            ValidatorBean.checkNotNull(item.getOutPartNo(), "产成零件号不能为空");
            ValidatorBean.checkNotNull(item.getEquipmentCode(), "设备不能为空");

            DdlPackBean seriesPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
            if (StringUtils.isEmpty(item.getInPartNo())) {
                DdlPreparedPack.getStringEqualPack(item.getInPartNo(), "inPartNo", seriesPackBean);
            }
            DdlPreparedPack.getStringEqualPack(item.getOutPartNo(), "outPartNo", seriesPackBean);
            DdlPreparedPack.getStringEqualPack(item.getEquipmentCode(), "equipmentCode", seriesPackBean);


            boolean flg = baseRDao.isExitByHql(seriesPackBean);
            if (flg) {
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                        .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                        .setErrorDetail("该非排序加工规则已存在，请检查数据")
                        .build();
            }
        }
    }
}
