package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base;

import cn.estsh.i3plus.ext.mes.api.base.IMesLabelTemplateService;
import cn.estsh.i3plus.ext.mes.api.base.IMesWorkOrderCutService;
import cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.print.IPrintTemplateStrategyService;
import cn.estsh.i3plus.ext.mes.apiservice.utils.MesException;
import cn.estsh.i3plus.ext.mes.pojo.model.MesProduceSnPrintModel;
import cn.estsh.i3plus.ext.mes.pojo.util.MesExtConstWords;
import cn.estsh.i3plus.mes.api.iservice.busi.ISyncFuncService;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.platform.common.tool.MathOperation;
import cn.estsh.i3plus.platform.common.tool.TimeTool;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.common.PagerHelper;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.*;
import cn.estsh.i3plus.pojo.mes.model.GenSerialNoModel;
import cn.estsh.i3plus.pojo.mes.repository.*;
import cn.estsh.i3plus.pojo.mes.util.MesExtEnumUtil;
import cn.estsh.impp.framework.boot.util.SpringContextsUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

@Service
@Slf4j
public class MesWorkOrderCutService extends BaseMesService<MesWorkOrderCut> implements IMesWorkOrderCutService {

    @Autowired
    private ISyncFuncService syncFuncService;

    @Autowired
    private MesWorkOrderCutRepository workOrderCutRepository;

    @Autowired
    private MesWorkOrderRepository workOrderRepository;

    @Autowired
    private MesCutSchemeFgRepository cutSchemeFgRepository;

    @Autowired
    private MesCutSchemeRepository cutSchemeRepository;

    @Autowired
    private MesCutSchemeMaterialRepository cutSchemeMaterialRepository;

    @Autowired
    private MesWorkOrderCutMaterialRepository workOrderCutMaterialRepository;

    @Autowired
    private MesWorkOrderCutDetailRepository workOrderCutDetailRepository;

    @Autowired
    private IMesPrintedSnLogRepository printedSnLogRepository;

    @Autowired
    private IMesLabelTemplateService mesLabelTemplateService;

    protected void setPackQueryBean(MesWorkOrderCut bean, DdlPackBean packBean) {
        DdlPreparedPack.getStringLikerPack(bean.getCutWorkOrderNo(), MesExtConstWords.CUT_WORK_ORDER_NO, packBean);
        DdlPreparedPack.getStringLikerPack(bean.getCutCode(), MesExtConstWords.CUT_CODE, packBean);
        if (!StringUtils.isEmpty(bean.getCreateDateTimeStart()) || !StringUtils.isEmpty(bean.getCreateDateTimeEnd())) {
            DdlPreparedPack.timeBuilder(
                    bean.getCreateDateTimeStart(),
                    bean.getCreateDateTimeEnd(),
                    MesExtConstWords.CREATE_DATE_TIME, packBean, true);
        }
        DdlPreparedPack.getNumEqualPack(bean.getWorkOrderStatus(), MesExtConstWords.WORK_ORDER_STATUS, packBean);
    }

    @Override
    public void doGenerateCutWorkOrder(List<Long> workOrderIds, String cutCode, String remark, String organizeCode,
                                       String userName, Integer editionNumber) {

        List<MesWorkOrder> workOrderList = getMesWorkOrders(workOrderIds, organizeCode);

        Map<String, List<MesWorkOrder>> workOrderListMapByPartNo = workOrderList.stream().collect(Collectors.groupingBy(MesWorkOrder::getPartNo));

        MesCutScheme cutScheme = getMesCutScheme(cutCode, organizeCode);

        List<MesCutSchemeMaterial> cutSchemeMaterialList = getMesCutSchemeMaterials(cutCode, organizeCode);

        List<MesCutSchemeFg> cutSchemeFgList = getMesCutSchemeFgs(cutCode, organizeCode);

        // 校验工单物料是否已在成品配置中进行维护
        checkWorkOrderPartValid(workOrderList, cutSchemeFgList);

        // 裁片工单生成版数
        editionNumber = editionNumber == null ? 1 : editionNumber;

        // 记录工单剩余未完成数量
        Map<String, Double> workOrderUnCompleteMap = new HashMap<>();
        // 记录当前裁片工单 每个工单的计划数量
        Map<String, Double> workOrderQtyMap = new HashMap<>();
        List<MesWorkOrderCutDetail> workOrderCutDetailtList = new ArrayList<>();
        List<MesWorkOrderCut> workOrderCuttList = new ArrayList<>();
        List<MesWorkOrderCutMaterial> workOrderCutMaterialList = new ArrayList<>();

        // 校验工单未完成数量是否满足计划数量
        checkQtyIsEnough(editionNumber, workOrderListMapByPartNo, cutScheme, cutSchemeFgList, workOrderUnCompleteMap, workOrderQtyMap);

        // 预计生成裁片工单数量
        for (int count = 0; count < editionNumber; count++) {
            MesWorkOrderCut workOrderCut = new MesWorkOrderCut();
            ConvertBean.serviceModelInitialize(workOrderCut, userName);
            workOrderCut.setCutCode(cutScheme.getCutCode());
            workOrderCut.setCutName(cutScheme.getCutName());
            workOrderCut.setWorkCenterCode(cutScheme.getWorkCenterCode());
            workOrderCut.setWorkOrderStatus(CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue() ==  cutScheme.getIsAutoRelease() ? MesExtEnumUtil.CUT_ORDER_STATUS.RELEASED.getValue() :  MesExtEnumUtil.CUT_ORDER_STATUS.CREATE.getValue());
            workOrderCut.setRemark(remark);

            String cutWorkOrderNo = doGenerateSerialNo(cutScheme.getCutCode(), organizeCode, userName);

            log.info("裁片工单号:{}", cutWorkOrderNo);

            workOrderCut.setCutWorkOrderNo(cutWorkOrderNo);
            workOrderCut.setOrganizeCode(organizeCode);
            workOrderCut.setPrintStatus(MesExtEnumUtil.PRINT_STATUS.UNPRINT.getValue());
            workOrderCuttList.add(workOrderCut);
            
            for (MesCutSchemeMaterial cutSchemeMaterial : cutSchemeMaterialList) {
                MesWorkOrderCutMaterial workOrderCutMaterial = new MesWorkOrderCutMaterial();
                BeanUtils.copyProperties(cutSchemeMaterial, workOrderCutMaterial, MesExtConstWords.ID);
                ConvertBean.serviceModelInitialize(workOrderCutMaterial, userName);
                workOrderCutMaterial.setOrganizeCode(organizeCode);
                workOrderCutMaterial.setCutWorkOrderNo(workOrderCut.getCutWorkOrderNo());
                workOrderCutMaterial.setSystemSyncStatus(CommonEnumUtil.FALSE);
                workOrderCutMaterialList.add(workOrderCutMaterial);
            }

            for (MesCutSchemeFg cutSchemeFg : cutSchemeFgList) {
                List<MesWorkOrder> workOrders = workOrderListMapByPartNo.get(cutSchemeFg.getPartNo());
                for (MesWorkOrder workOrder : workOrders) {
                    MesWorkOrderCutDetail detail = new MesWorkOrderCutDetail();
                    ConvertBean.serviceModelInitialize(detail, userName);
                    detail.setOrganizeCode(organizeCode);
                    detail.setCutWorkOrderNo(cutWorkOrderNo);
                    detail.setPartNo(workOrder.getPartNo());
                    detail.setPartName(workOrder.getPartName());
                    detail.setWorkOrderNo(workOrder.getWorkOrderNo());
                    detail.setRemark(cutSchemeFg.getRemark());
                    detail.setQty(workOrderQtyMap.get(workOrder.getWorkOrderNo()));
                    workOrderCutDetailtList.add(detail);
                }
            }
        }

        workOrderCutMaterialRepository.saveAll(workOrderCutMaterialList);
        workOrderCutDetailRepository.saveAll(workOrderCutDetailtList);
        workOrderCutRepository.saveAll(workOrderCuttList);

        String nowTime = TimeTool.getNowTime(true);
        workOrderList.forEach(o -> {
            o.setUnCompleteQty(workOrderUnCompleteMap.get(o.getWorkOrderNo()));
            o.setModifyUser(userName);
            o.setModifyDatetime(nowTime);
            o.setSystemSyncStatus(CommonEnumUtil.FALSE);
        });

        workOrderRepository.saveAll(workOrderList);

    }

    @Override
    public ListPager<MesWorkOrderCutDetail> queryWorkOrderCutDetailByPager(MesWorkOrderCutDetail workOrderCutDetail, Pager pager) {
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(workOrderCutDetail.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(workOrderCutDetail.getCutWorkOrderNo(), MesExtConstWords.CUT_WORK_ORDER_NO, packBean);
        pager = PagerHelper.getPager(pager, workOrderCutDetailRepository.findByHqlWhereCount(packBean));
        List<MesWorkOrderCutDetail> list = workOrderCutDetailRepository.findByHqlWherePage(packBean, pager);
        return new ListPager<>(list, pager);
    }

    private static void checkQtyIsEnough(Integer editionNumber, Map<String, List<MesWorkOrder>> workOrderListMapByPartNo,
                                         MesCutScheme cutScheme, List<MesCutSchemeFg> cutSchemeFgList,
                                         Map<String, Double> workOrderUnCompleteMap, Map<String, Double> workOrderQtyMap) {
        for (MesCutSchemeFg cutSchemeFg : cutSchemeFgList) {

            List<MesWorkOrder> workOrders = workOrderListMapByPartNo.get(cutSchemeFg.getPartNo());

            //MesWorkOrder workOrder = workOrders.get(0);

            //if (workOrders.size() > 1)
            //    MesException.throwMesBusiException("零件号【%s】存在多个工单", cutSchemeFg.getPartNo());

            if (!CollectionUtils.isEmpty(workOrders)) workOrders.sort(Comparator.comparing(MesWorkOrder::getUnCompleteQty));

            // 成品配置计划数量 * 版数 = 裁片工单该成品总消耗数量
            Double totalQty = MathOperation.mul(cutSchemeFg.getQty(), editionNumber);
            Double srcTotalQty = totalQty.doubleValue();
            Double unCompleteQty = 0.0;
            List<String> needWorkOrderNoList = new ArrayList<>();
            for (MesWorkOrder order : workOrders) {

                if (!order.getWorkCenterCode().equals(cutScheme.getWorkCenterCode()))
                    MesException.throwMesBusiException("工单【%s】关联产线【%】与裁片方案【%】关联产线不一致!",
                            order.getWorkCenterCode(), cutScheme.getWorkCenterCode());

                if (unCompleteQty.compareTo(srcTotalQty) >= 0)
                    MesException.throwMesBusiException("零件号【%s】工单【%s】未完成数量汇总【%s】已满足产成总数【%s】!请取消勾选工单【%s】",
                            order.getWorkCenterCode(), needWorkOrderNoList, cutScheme.getWorkCenterCode(),
                            workOrders.subList(needWorkOrderNoList.size(), workOrders.size()));

                unCompleteQty = MathOperation.add(unCompleteQty, order.getUnCompleteQty());

                needWorkOrderNoList.add(order.getWorkOrderNo());

                if (order.getUnCompleteQty().compareTo(totalQty) >= 0) {
                    workOrderUnCompleteMap.put(order.getWorkOrderNo(), MathOperation.sub(order.getUnCompleteQty(), totalQty));
                    workOrderQtyMap.put(order.getWorkOrderNo(), totalQty);
                    totalQty = 0.0;
                } else {
                    workOrderUnCompleteMap.put(order.getWorkOrderNo(), 0.0);
                    workOrderQtyMap.put(order.getWorkOrderNo(), order.getUnCompleteQty());
                    totalQty = MathOperation.sub(totalQty, order.getUnCompleteQty());
                }

            }

            if (totalQty.compareTo(unCompleteQty) > 0) {
                List<String> workOrderNoList = workOrders.stream().map(MesWorkOrder::getWorkOrderNo).collect(Collectors.toList());
                MesException.throwMesBusiException("零件号【%s】工单【%s】未完成数量【%s】不满足产成总数【%s】!",
                        cutSchemeFg.getPartNo(), workOrderNoList, unCompleteQty, totalQty);
            }

        }
    }

    private List<MesWorkOrder> getMesWorkOrders(List<Long> workOrderIds, String organizeCode) {
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getInPackList(workOrderIds, MesExtConstWords.ID, packBean);
        List<MesWorkOrder> workOrderList = workOrderRepository.findByHqlWhere(packBean);

        if (CollectionUtils.isEmpty(workOrderList)) {
            MesException.throwMesBusiException("工单信息不存在");
        }

        List<Long> workOrderIdDbList = workOrderList.stream().map(MesWorkOrder::getId).collect(Collectors.toList());

        for (Long workOrderId : workOrderIds) {
            if (!workOrderIdDbList.contains(workOrderId)) {
                MesException.throwMesBusiException("工单信息ID【%s】不存在", workOrderId);
            }
        }

        for (MesWorkOrder mesWorkOrder : workOrderList) {
            if (mesWorkOrder.getWorkOrderStatus() != MesExtEnumUtil.ORDER_STATUS.RELEASE.getValue() && mesWorkOrder.getWorkOrderStatus() != MesExtEnumUtil.ORDER_STATUS.PROCESS.getValue()) {
                MesException.throwMesBusiException("工单【%s】状态【%s】无法生成裁片工单", mesWorkOrder.getWorkOrderNo(), MesExtEnumUtil.ORDER_STATUS.valueOfDescription(mesWorkOrder.getWorkOrderStatus()));
            }
        }
        return workOrderList;
    }

    private MesCutScheme getMesCutScheme(String cutCode, String organizeCode) {
        MesCutScheme cutScheme = getCutScheme(cutCode, organizeCode);

        if (cutScheme == null)
            MesException.throwMesBusiException("裁片方案【%s】无效", cutCode);
        return cutScheme;
    }

    private static void checkWorkOrderPartValid(List<MesWorkOrder> workOrderList, List<MesCutSchemeFg> cutSchemeFgList) {
        List<String> partNoList = workOrderList.stream().map(MesWorkOrder::getPartNo).distinct().collect(Collectors.toList());
        List<String> fgPartNoList = cutSchemeFgList.stream().map(MesCutSchemeFg::getPartNo).distinct().collect(Collectors.toList());

        partNoList.removeAll(fgPartNoList);

        if (!CollectionUtils.isEmpty(partNoList)) MesException.throwMesBusiException("零件号【%s】未在裁片方案成品配置中维护", partNoList);
    }

    private List<MesCutSchemeFg> getMesCutSchemeFgs(String cutCode, String organizeCode) {
        List<MesCutSchemeFg> cutSchemeFgList = getCutSchemeFgList(cutCode, organizeCode);

        if (CollectionUtils.isEmpty(cutSchemeFgList)) MesException.throwMesBusiException("裁片方案【%s】缺少裁片方案成品配置", cutCode);
        return cutSchemeFgList;
    }

    private List<MesCutSchemeMaterial> getMesCutSchemeMaterials(String cutCode, String organizeCode) {
        List<MesCutSchemeMaterial> cutSchemeMaterialList = getCutSchemeMaterialList(cutCode, organizeCode);

        if (CollectionUtils.isEmpty(cutSchemeMaterialList)) MesException.throwMesBusiException("裁片方案【%s】缺少原材料配置", cutCode);
        return cutSchemeMaterialList;
    }

    @Override
    public void doReleaseCutWorkOrder(List<String> cutWorkOrderNos, String organizeCode, String userName) {

        if (CollectionUtils.isEmpty(cutWorkOrderNos)) MesException.throwMesBusiException("裁片工单为空");

        List<MesWorkOrderCut> workOrderCuts = new ArrayList<>();

        for (String cutWorkOrderNo : cutWorkOrderNos) {

            if (StringUtils.isEmpty(cutWorkOrderNos)) MesException.throwMesBusiException("裁片工单为空");

            DdlPackBean packBean = DdlPackBean.getDdlPackBean(organizeCode);
            DdlPreparedPack.getStringEqualPack(cutWorkOrderNo, MesExtConstWords.CUT_WORK_ORDER_NO, packBean);
            MesWorkOrderCut workOrderCutDb = workOrderCutRepository.getByProperty(packBean);

            if (workOrderCutDb == null) MesException.throwMesBusiException("裁片工单不存在", cutWorkOrderNo);

            if (workOrderCutDb.getWorkOrderStatus() != MesExtEnumUtil.CUT_ORDER_STATUS.CREATE.getValue()) MesException.throwMesBusiException("裁片工单【%】状态不为【%s】,当前状态为【%s】", cutWorkOrderNo, MesExtEnumUtil.CUT_ORDER_STATUS.RELEASED.getValue(), MesExtEnumUtil.CUT_ORDER_STATUS.valueOfDescription(workOrderCutDb.getWorkOrderStatus()));

            workOrderCuts.add(workOrderCutDb);

        }

        for (MesWorkOrderCut workOrderCut : workOrderCuts) {
            workOrderCut.setWorkOrderStatus(MesExtEnumUtil.CUT_ORDER_STATUS.RELEASED.getValue());
            ConvertBean.serviceModelUpdate(workOrderCut, userName);
        }

        workOrderCutRepository.saveAll(workOrderCuts);
    }

    @Override
    public List<Map<String, Object>> doPrintCutWorkOrder(List<String> cutWorkOrderNos, String organizeCode, String userName) {

        DdlPackBean packBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getInPackList(cutWorkOrderNos, MesExtConstWords.CUT_WORK_ORDER_NO, packBean);
        List<MesWorkOrderCut> workOrderCutDbList = workOrderCutRepository.findByHqlWhere(packBean);

        List<String> cutWorkOrderNoDbList = workOrderCutDbList.stream().map(MesWorkOrderCut::getCutWorkOrderNo)
                .distinct().collect(Collectors.toList());

        if (CollectionUtils.isEmpty(cutWorkOrderNoDbList)) MesException.throwMesBusiException("裁片工单不存在");

        cutWorkOrderNos = new ArrayList<>(cutWorkOrderNos);

        cutWorkOrderNos.removeAll(cutWorkOrderNoDbList);

        if (!CollectionUtils.isEmpty(cutWorkOrderNos))
            MesException.throwMesBusiException("裁片工单【%s】不存在", cutWorkOrderNos);

        workOrderCutDbList.forEach(o -> {
            if (o.getPrintStatus() == MesExtEnumUtil.PRINT_STATUS.PRINTED.getValue())
                MesException.throwMesBusiException("裁片工单【%s】已打印过标签，无法再次打印", o.getCutWorkOrderNo());
        });

        // 返回结果
        List<Map<String, Object>> resultMapList = new ArrayList<>();

        List<String> cutCodeList = workOrderCutDbList.stream().map(MesWorkOrderCut::getCutCode)
                .distinct().collect(Collectors.toList());

        // 根据裁片方案代码分组裁片工单
        Map<String, List<MesWorkOrderCut>> workOrderCutMapByCutCode = workOrderCutDbList.stream().collect(
                Collectors.groupingBy(MesWorkOrderCut::getCutCode));

        // 获取裁片方案
        List<MesCutScheme> mesCutSchemes = queryCutSchemeList(cutCodeList, organizeCode);

        // 根据裁片方案代码分组
        Map<String, MesCutScheme> cutSchemeMapByTemplate = mesCutSchemes.stream().collect(
                Collectors.toMap(MesCutScheme::getTemplateCode, Function.identity(), (x, y) -> y));

        for (Map.Entry<String, MesCutScheme> entry : cutSchemeMapByTemplate.entrySet()) {

            MesCutScheme cutScheme = entry.getValue();
            List<MesWorkOrderCut> workOrderCutList = workOrderCutMapByCutCode.get(cutScheme.getCutCode());

            MesLabelTemplate labelTemplate = mesLabelTemplateService.getLabelTemplate(cutScheme.getTemplateCode(), organizeCode);

            //模板信息丢失抛出异常
            if (labelTemplate == null)
                MesException.throwBusiException("请检查裁片方案,裁片方案[%s]模板代码[%s]无效!", cutScheme.getCutCode(), cutScheme.getTemplateCode());

            //根据反射获取策略类--封装打印数据
            String methodCode = labelTemplate.getMethodCode();

            //模板信息丢失抛出异常
            if (StringUtils.isEmpty(methodCode))
                MesException.throwBusiException("裁片方案[%s]模板代码[%s]方法类为空,请配置!", cutScheme.getCutCode(), labelTemplate.getTemplateCode());

            MesProduceSnPrintModel mesProduceSnPrintModel = new MesProduceSnPrintModel();
            mesProduceSnPrintModel.setMesLabelTemplate(labelTemplate);
            mesProduceSnPrintModel.setSourceData(workOrderCutList);
            mesProduceSnPrintModel.setPrinter(cutScheme.getPrinter());

            IPrintTemplateStrategyService strategyService = (IPrintTemplateStrategyService) SpringContextsUtil.getBean(methodCode);
            MesProduceSnPrintModel printModel = strategyService.execute(null, mesProduceSnPrintModel, null);

            resultMapList.addAll(printModel.getPrintContextList());

        }

        List<MesPrintedSnLog> printedSnLogList = new ArrayList<>();
        workOrderCutDbList.forEach(o -> {
            o.setPrintStatus(MesExtEnumUtil.PRINT_STATUS.PRINTED.getValue());
            o.setPrintTime(TimeTool.getNowTime(true));
            ConvertBean.serviceModelUpdate(o, userName);
            o.setSystemSyncStatus(CommonEnumUtil.FALSE);

            MesPrintedSnLog printedSnLog = new MesPrintedSnLog();
            printedSnLog.setOrganizeCode(organizeCode);
            printedSnLog.setCutWorkOrderNo(o.getCutWorkOrderNo());
            printedSnLog.setBarcode(o.getCutWorkOrderNo());
            printedSnLog.setProductDate(TimeTool.getNowTime(MesExtConstWords.DATE_FORMAT));
            ConvertBean.serviceModelInitialize(printedSnLog, userName);
            printedSnLogList.add(printedSnLog);

        });

        workOrderCutRepository.saveAll(workOrderCutDbList);

        printedSnLogRepository.saveAll(printedSnLogList);


        return resultMapList;

    }

    @Override
    public void doUpdatePrintStatus(List<String> cutWorkOrderIds, String organizeCode, String userName) {

        for (String cutWorkOrderId : cutWorkOrderIds) {
            workOrderCutRepository.updateByProperties(
                    new String[]{MesExtConstWords.ID, MesExtConstWords.ORGANIZE_CODE},
                    new Object[]{Long.parseLong(cutWorkOrderId), organizeCode},
                    new String[]{MesExtConstWords.IS_DELETED, MesExtConstWords.MODIFY_USER, MesExtConstWords.MODIFY_DATE_TIME, MesExtConstWords.SYSTEM_SYNC_STATUS, MesExtConstWords.PRINT_STATUS},
                    new Object[]{CommonEnumUtil.VALID,userName,TimeTool.getNowTime(true), CommonEnumUtil.FALSE, MesExtEnumUtil.PRINT_STATUS.UNPRINT.getValue()});
        }

    }

    private String doGenerateSerialNo(String cutCode, String organizeCode, String userName) {
        return syncFuncService.syncSerialNo(
                new GenSerialNoModel(MesExtConstWords.CUT_WORK_ORDER_NO_FUL).dynamicRule(cutCode).year(TimeTool.pareDateToString(MesExtConstWords.DATE_FORMAT_YY, new Date())).organizeCode(organizeCode),
                userName, organizeCode, 1).getResultList().get(0).toString();
    }

    private MesCutScheme getCutScheme(String cutCode, String organizeCode) {

        DdlPackBean packBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getStringEqualPack(cutCode, MesExtConstWords.CUT_CODE, packBean);
        return cutSchemeRepository.getByProperty(packBean);

    }

    private List<MesCutScheme> queryCutSchemeList(List<String> cutCodeList, String organizeCode) {

        DdlPackBean packBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getInPackList(cutCodeList, MesExtConstWords.CUT_CODE, packBean);
        return cutSchemeRepository.findByHqlWhere(packBean);

    }

    private List<MesCutSchemeFg> getCutSchemeFgList(String cutCode, String organizeCode) {
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getStringEqualPack(cutCode, MesExtConstWords.CUT_CODE, packBean);
        return cutSchemeFgRepository.findByHqlWhere(packBean);
    }

    private List<MesCutSchemeMaterial> getCutSchemeMaterialList(String cutCode, String organizeCode) {
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getStringEqualPack(cutCode, MesExtConstWords.CUT_CODE, packBean);
        return cutSchemeMaterialRepository.findByHqlWhere(packBean);
    }
}
