package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.busi;

import cn.estsh.i3plus.ext.mes.api.busi.IMesCreateWorkOrder;
import cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base.MesBomService;
import cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base.MesPartSapService;
import cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base.MesWorkOrderService;
import cn.estsh.i3plus.mes.api.iservice.busi.ISyncFuncService;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.platform.common.tool.TimeTool;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.*;
import cn.estsh.i3plus.pojo.mes.bean.shipping.MesCustSortInfo;
import cn.estsh.i3plus.pojo.mes.model.GenSerialNoModel;
import cn.estsh.i3plus.pojo.mes.repository.*;
import cn.estsh.i3plus.pojo.mes.util.MesExtEnumUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

@Service
@Slf4j
public class MesCreateWorkOrderImpl implements IMesCreateWorkOrder {
    @Autowired
    private MesPartSapRepository mesPartSapRao;

    @Autowired
    private MesProductVersionRepository mesProductVersionRao;

    @Autowired
    private MesWorkOrderRepository mesWorkOrderRao;


    @Autowired
    private MesShiftRepository mesShiftRao;


    @Autowired
    private MesBomService mesBomService;

    @Autowired
    private MesBomRepository bomRao;

    @Autowired
    private MesProdRuleSortCfgRepository mesProdRuleSortCfgRao;

    @Autowired
    private MesWorkOrderPartRepository workOrderPartRao;

    @Autowired
    private MesWorkOrderAssemblyRepository workOrderAssemblyRao;

    @Autowired
    private ISyncFuncService syncFuncService;


    @Autowired
    private MesPartPtrRepository partPtrRao;

    @Autowired
    private MesPartSapService mesPartSapService;

    @Autowired
    private MesProduceSnRepository mesProduceSnRao;

    @Autowired
    private MesQueueOrderRepository queueOrderRao;

    @Autowired
    private MesWorkOrderService mesWorkOrderService;

    @Autowired
    private  MesWorkCenterRepository workCenterRao;

    @Autowired
    private MesPartPtrDetailRepository partPtrDetailRao;

    @Autowired
    private MesCustomerCarModelRepository mesCustomerCarModelRepository;


    @Transactional(propagation = Propagation.REQUIRES_NEW)
    public void createWorkOrder(MesPartProdGroup partProdGroup, MesPartProdGroupDetail mesPartProdGroupDetail, MesPartSap partSap, MesCustSortInfo sortInfo,Long productSeq) {

        //判断vin是否已经生成了S单,如果已经生成S单 则不再生成工单
        String vinCode = sortInfo.getVinCode();
        DdlPackBean existPackBean = DdlPackBean.getDdlPackBean(sortInfo.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(vinCode, "vinCode", existPackBean);
//        DdlPreparedPack.getStringEqualPack(MesExtEnumUtil.ORDER_TYPE_IDENTIFICATION.S.getValue(),"orderFlag",existPackBean);
        List<MesWorkOrder> workOrderList = mesWorkOrderRao.findByHqlWhere(existPackBean);
        if (!workOrderList.isEmpty()) {
            MesWorkOrder mesWorkOrder = workOrderList.get(0);
            if (mesWorkOrder.getOrderFlag().equals(MesExtEnumUtil.ORDER_TYPE_IDENTIFICATION.S.getValue())) {
                sortInfo.setDescription("当前vin号已存在S单!");
                return;
            }
            sortInfo.setDescription("当前vin号已经解析过!");
            return;
        }

        String organizeCode = partProdGroup.getOrganizeCode();
        MesWorkOrder mesWorkOrder = new MesWorkOrder();
        mesWorkOrder.setCustCode(partProdGroup.getCustCode());
        //工单版本号
        mesWorkOrder.setProductVersion("1");
        //工单类型
        mesWorkOrder.setWorkOrderType(MesExtEnumUtil.ORDER_TYPE.SORT.getValue());
        //订单序号=客户排序信息的序号
        mesWorkOrder.setWorkOrderSeq(sortInfo.getCustInfoSeq().toString());
        mesWorkOrder.setOrganizeCode(sortInfo.getOrganizeCode());
        mesWorkOrder.setWorkCenterCode(partProdGroup.getWorkCenterCode());
        //根据产线查找区域代码
        DdlPackBean centerPackBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getStringEqualPack(partProdGroup.getWorkCenterCode(), "workCenterCode", centerPackBean);
        List<MesWorkCenter> workCenterList = workCenterRao.findByHqlTopWhere(centerPackBean, 1);
        if (!workCenterList.isEmpty()) {
            mesWorkOrder.setAreaCode(workCenterList.get(0).getAreaCode());
        }
        mesWorkOrder.setWorkCellCode(mesPartProdGroupDetail.getWorkCellCode());

        //来源
        mesWorkOrder.setWorkOrderSource(sortInfo.getId().toString());

        mesWorkOrder.setVinCode(sortInfo.getVinCode());
        mesWorkOrder.setProductTime(sortInfo.getMatchTime());
        mesWorkOrder.setProduceTime(sortInfo.getMatchTime());
        mesWorkOrder.setPlanStartTime(sortInfo.getMatchTime());
        mesWorkOrder.setPlanEndTime(sortInfo.getInfoPointTime());
        mesWorkOrder.setPtrInfo(sortInfo.getRemark());
        //客户订单号
        mesWorkOrder.setCustOrderNo(sortInfo.getCustOrderCode());
        //车型代码
//        mesWorkOrder.setCarModelCode(sortInfo.getVehicleCategory());
        //车型描述 mesWorkOrder.setCarName = vehicleClassCode

        //备注取自于车型配置表中
        //todo 根据VehicleNo车型代码，sortInfo.custCode,sortInfo.CustPlantCode,organizeCode 查询表TI_CIM_VEHICLE_MODEL的 remark 作为从车型配置表中获取打印备注信息

        mesWorkOrder.setCustPartNo(sortInfo.getCustPartNo());
        // 11. 是否将生成日期减一天
        // 班次 根据产线+计划时间匹配班次
        DdlPackBean shiftPackBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getStringEqualPack(mesPartProdGroupDetail.getWorkCenterCode(), "workCenterCode", shiftPackBean);
        List<MesShift> shiftList = mesShiftRao.findByHqlWhere(shiftPackBean);
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        Date date = new Date();
        try {
            date = sdf.parse(sortInfo.getMatchTime());
        } catch (ParseException e) {
            log.error("解析时间报错,无法转换成时间，matchTime:{}", sortInfo.getMatchTime());
        }
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        int hours = calendar.get(Calendar.HOUR_OF_DAY);

        int minutes = calendar.get(Calendar.MINUTE);
        long shiftTime = Long.parseLong(String.format("%02d:%02d:00", hours, minutes).replaceAll(":", ""));
        if (!shiftList.isEmpty()) {
            for (MesShift shift : shiftList) {
                long startTime = Long.parseLong(shift.getStartTime().replaceAll(":", ""));
                long endTime = Long.parseLong(shift.getEndTime().replaceAll(":", ""));
                if (startTime < endTime) {
                    if (shiftTime >= startTime && shiftTime < endTime) {
                        mesWorkOrder.setShiftName(shift.getShiftName());
                        mesWorkOrder.setShiftCode(shift.getShiftCode());
                    }
                } else {
                    if (shiftTime >= startTime || shiftTime < endTime) {
                        mesWorkOrder.setShiftName(shift.getShiftName());
                        mesWorkOrder.setShiftCode(shift.getShiftCode());
                    }
                }
            }
        } else {
            log.info("当前产线:{} 没有配置班次", mesPartProdGroupDetail.getWorkCenterCode());
        }
        //零件相关
        mesWorkOrder.setPartNo(partSap.getPartNo());
        mesWorkOrder.setPartName(partSap.getPartName());

        //工单号生成规则 【工厂号+yyyyMMdd+5位流水】

        // 13.生产版本 organizeCode + partNo
        /**
         * 生产版本是根据 生产零检号查询，不是根据erp零件号查询
         */
        MesProductVersion prodVersion = getProdVersion(organizeCode, mesPartProdGroupDetail.getProductPartNo());
        if (prodVersion == null) {
            log.info("请配置工厂:{},partNo:{}的生产版本", mesPartProdGroupDetail.organizeCode, mesPartProdGroupDetail.getProductPartNo());
        } else {
            String productVersion = prodVersion.getProductVersion();
            mesWorkOrder.setProductVersion(productVersion);
        }

        // bom prodVersion.alternativePartList bom版本
        /**
         * bom也需要根据生产零件号拆分
         */
        MesBom bom = mesBomService.findBom(organizeCode, prodVersion == null ? null : prodVersion.getAlternativePartList(), mesPartProdGroupDetail.getProductPartNo());
        if (bom != null) {
            mesWorkOrder.setBomCode(bom.getBomCode());
        } else {
            log.info("工厂:{},生产零件号:{}没有维护bom", organizeCode, mesPartProdGroupDetail.getProductPartNo());
            return;
        }

        mesWorkOrder.setQty(sortInfo.getQty());
        mesWorkOrder.setUnCompleteQty(sortInfo.getQty());
        mesWorkOrder.setUnit(partSap.getUnit());

        if (partProdGroup.getIsAutoRelease() == CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue()) {
            mesWorkOrder.setWorkOrderStatus(MesExtEnumUtil.ORDER_STATUS.RELEASE.getValue());
        } else {
            mesWorkOrder.setWorkOrderStatus(MesExtEnumUtil.ORDER_STATUS.CREATE.getValue());
        }

        mesWorkOrder.setPartProdGroupCode(partProdGroup.getPartProdGroupCode());
        mesWorkOrder.setCarModelCode(sortInfo.getVehicleNo());
        mesWorkOrder.setFid(UUID.randomUUID().toString());
        mesWorkOrder.setQcStatus(MesExtEnumUtil.PRODUCE_QC_STATUS.QUALIFIED.getValue());
        /**
         * RS 为记录分隔符ASCII:30
         * GS 为分组符ASCII:29
         * EOT 为传输结束符ASCII:04
         * {splitRule} [)>%RS%06%GS%Y
         * {splitRule} vpps= 4110000000000X
         * {splitRule} %GS%P
         * {partNo} 8位客户零件号
         * {splitRule} %GS%12V
         * {splitRule} duns 545238347
         * {splitRule} %GS%T1A
         * {yydayofyear}
         * {splitRule} A2B4C
         * {serialNo} 4位 步长1
         * {splitRule}%RS%%EOT%
         *{spiltRule}{spiltRule}{spiltRule}{partNo}{spiltRule}{spiltRule}{spiltRule}{year}{day}{spiltRule}{serialNo}{spiltRule}
         * [)>%RS%06%GS%Y,4110000000000X,%GS%P,%GS%12V,545238347,%GS%T1A,A2B4C,%RS%%EOT%
         *  通用二维码组成部分('RS'为记录分隔符ASCII:30，'GS'为分组符ASCII:29,'EOT'为传输结束符ASCII:04)
         *  [)>+'RS'+06+'GS'+Y+vpps+'GS'+P+8位客户零件号+'GS'+12V+DUNS+'GS'+T+1位line num+1位班次号+2位年份+3位天数+1位追溯(精确追溯A，批次追溯@)+2工程更改记录+2位自定义+4位流水号+'RS'+'EOT'
         *  如：[)>+'RS'+06+'GS'+Y+4110000000000X+'GS'+P+12345678+'GS'+12V+545295227+'GS'+T+1A13304A2B4C0001+'RS'+'EOT'
         *
         *  vpps = Y00000000000000
         *  8位客户零件号=最后四位要大写
         *  Duns是什么？
         *  1位lineNumber是什么？ 固定值 1
         *  1位班次号是什么？
         *  2位年YY 24
         *  3位天-~365/366
         *  追溯
         *  A--精确追溯
         *  @--批次追溯
         *
         *  2位工程更改记录
         *  2位供应商自定义信息
         *  4位流水号，当班班次计数，每班班次归零
         *
         *  WH_CUST_SN_RULE
         */

        Calendar custCalendar = Calendar.getInstance();
        custCalendar.setTime(new Date());
        GenSerialNoModel custGenSerialNoModel = new GenSerialNoModel("WH_CUST_SN_RULE");
        String partNo = mesWorkOrder.getCustPartNo();
        String prefix = partNo.substring(0, partNo.length() - 4);
        String partNo4 = partNo.substring(partNo.length() - 4).toUpperCase();
        custGenSerialNoModel.setPartNo(prefix + partNo4);
        String year = custCalendar.get(Calendar.YEAR) + "";
        custGenSerialNoModel.setYear(year.substring(2));
        custGenSerialNoModel.setDay(String.format("%03d", custCalendar.get(Calendar.DAY_OF_YEAR)));
        List<String> custNumRuleList = syncFuncService.syncSerialNo(custGenSerialNoModel, "edi", organizeCode, 1).getResultList();
        // [)>%RS%06%GS%Y,4110000000000X,%GS%P,%GS%12V,545238347,%GS%T1A,A2B4C,%RS%%EOT%
        //截取替换为ascii码
        String snOriginal = custNumRuleList.get(0);
        String[] split = snOriginal.split("%RS%");
        StringBuilder rsStr = new StringBuilder();
        for (int i = 0; i < split.length - 1; i++) {
            rsStr.append(split[i]).append((char) 30);
        }
        rsStr.append(split[split.length - 1]);
        StringBuilder gsStr = new StringBuilder();
        String[] split1 = rsStr.toString().split("%GS%");
        for (int i = 0; i < split1.length - 1; i++) {
            gsStr.append(split1[i]).append((char) 29);
        }
        gsStr.append(split1[split1.length - 1]);
        String[] split2 = gsStr.toString().split("%EOT%");
        String sn = split2[0] + (char) 4;
        //保存条码
        mesWorkOrder.setCustSn(sn);

        // 打散bom
        List<MesWorkOrderPart> workOrderPartList = new ArrayList<>();
        if (partProdGroup.getIsDisassembleBom() == CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue()) {
            //打散bom使用生产零件号
            workOrderPartList = dismantleBom(mesPartProdGroupDetail.getProductPartNo(), mesWorkOrder.getQty(), sortInfo.getMatchTime(), bom.getBomCode(), sortInfo.getOrganizeCode(), mesPartProdGroupDetail.getWorkCellCode(), mesWorkOrder.getWorkOrderNo());
        }



        /**
         * 总成拆bom,变成子零件
         * 替换子零件，通过替换后的零件，去找排序工艺
         *
         * PTR逻辑 3种场景互斥
         * 1.如果选择了客户发送PTR信息，遇到PTR info直接转换单
         * 2.如果选择了内部PTR，需要看总成是否需要替换，生成F单
         * 3.如果两个都没有选择PTR,需要看总成是否需要替换 生成P单
         *
         * isCustomerSendPrtInfo 1是客户发送ptr
         * isInterPrt 1是内部ptr
         */
//        Map<String, String> ptrRuleListMap = new HashMap<>();
        //是否修改为P单
        boolean isModifyPOrder = false;
        boolean isFOrder = false;
        //查询mes_part_ptr表，根据物料清单查找ptr替换信息
        /**
         * PTR生效
         * 1.有开始时间和结束时间 和 需要数量，达到其中一个就结束
         * 2.有开始时间无结束时间 和 需要数量 达到需要的数量结束
         * 3.有开始时间无结束时间，无结束数量 一直替换
         */
        /**
         * 2024/11/16
         * PTR 分三种场景 --- mes_part_ptr
         * 1.基于排序加工规则 mes_part_ptr 中 create_by_assembly_setting  1 true --暂时没有使用
         * 替换零件及替换装配工位
         *
         * 2.不基于排序加工规则  2 false
         * 替换的零件装配工位不变,只改变零件信息
         *
         * 3.是否客户发送 is_cust_send_info  is_customer_send_prt_info = 1 true
         *  遇到就替换，不用管ptrPart的总成
         */
        //打印目视单使用
        StringBuilder descriptionStr = new StringBuilder();

        //是否基于排序加工规则
        // boolean isCreateByAssemblySetting=false;

        //客户ptr信息
        StringBuilder ptrInfos = new StringBuilder();
        String ptrInfo = mesWorkOrder.getPtrInfo();
        List<String> custPtrPartList = Arrays.asList(ptrInfo.split("\\|"));
        boolean ptrFlag = false;
        boolean fOrderFlag = false;
        List<MesWorkOrderAssembly> assemblyList = new ArrayList<>();
        if (!workOrderPartList.isEmpty()) {
            //是否基于排序加工规则--待定
            //查询装配目视项
            assemblyList = getAssemblyList(mesWorkOrder.getWorkCenterCode(), mesWorkOrder.getCarModelCode(), workOrderPartList, partProdGroup);
            //ptr信息
            List<String> partNoList = workOrderPartList.stream().map(MesWorkOrderPart::getPartNo).collect(Collectors.toList());
            List<MesPartPtr> partPtrList = getPartPtrList(organizeCode, mesWorkOrder.getWorkCenterCode(), partNoList);
            //遍历workOrderPartList

            for (MesWorkOrderPart orderPart : workOrderPartList) {
                List<MesPartPtr> ptr = partPtrList.stream().filter(item -> item.getSrcPartNo().equals(orderPart.getPartNo())).collect(Collectors.toList());
                if (ptr.isEmpty()) {
                    continue;
                }
                //判断是否是客户替换
                boolean isCust = false;
                for (MesPartPtr mesPartPtr : ptr) {
                    if (custPtrPartList.contains(mesPartPtr.getSrcPartNo())) {
                        isCust = true;
                        break;
                    }
                }

                MesPartPtr mesPartPtr = null;
                MesPartPtrDetail mesPartPtrDetail = null;

                if (isCust) {
                    //查找客户发送的ptr信息
                    List<MesPartPtr> custPartPtrList = ptr.stream().filter(item -> item.getIsCustomerSendPrtInfo() == CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue() && custPtrPartList.contains(item.getSrcPartNo())).collect(Collectors.toList());
                    if (!custPartPtrList.isEmpty()) {
                        mesPartPtr = custPartPtrList.get(0);
                    }
                } else {
                    //获取相同总成的ptrId 在ptr详情中
                    List<MesPartPtr> custPartPtrList = ptr.stream().filter(item -> item.getIsCustomerSendPrtInfo() == null || item.getIsCustomerSendPrtInfo() != CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue()).collect(Collectors.toList());
                    if (!custPartPtrList.isEmpty()) {
                        mesPartPtr = custPartPtrList.get(0);
                        //查询ptr总成--只有特定总成才会替换
                        DdlPackBean detailPartPtr = DdlPackBean.getDdlPackBean(organizeCode);
                        DdlPreparedPack.getNumEqualPack(mesPartPtr.getId(), "ptrId", detailPartPtr);
                        DdlPreparedPack.getStringEqualPack(mesWorkOrder.getPartNo(), "totalPartNo", detailPartPtr);
                        List<MesPartPtrDetail> ptrDetailList = partPtrDetailRao.findByHqlWhere(detailPartPtr);
                        if (!ptrDetailList.isEmpty()) {
                            mesPartPtrDetail = ptrDetailList.get(0);
                        }
                    }
                }

                if (!isCust && mesPartPtrDetail == null) {
                    continue;
                }

                ptrFlag = true;
                if (!isCust && mesPartPtr.getIsInterPrt()==CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue()){
                    fOrderFlag = true;
                }
                //将工单类零件表替换位PTR条件中维护的替换零件
                MesPartSap mesPartSap = mesPartSapService.getMesPartSapByPartNo(mesPartPtr.getDestPartNo(), organizeCode);
                orderPart.setPartNo(mesPartSap.getPartNo());
                orderPart.setPartName(mesPartSap.getPartName());
                //用于打印目视单
                descriptionStr.append(mesPartPtr.getMemo());
                //更新mes_part_ptr的实际数量 + 1
                mesPartPtr.setPrtVehicleActualCount(mesPartPtr.getPrtVehicleActualCount() + 1);
                ConvertBean.saveOrUpdate(mesPartPtr, "JOB");
                partPtrRao.update(mesPartPtr);
                if (mesPartPtrDetail != null) {
                    mesPartPtrDetail.setQty(mesPartPtrDetail.getQty() + 1);
                    ConvertBean.saveOrUpdate(mesPartPtrDetail, "JOB");
                    partPtrDetailRao.update(mesPartPtrDetail);
                }
                MesWorkOrderAssembly orderAssemblyList = null;
                for (MesWorkOrderAssembly orderAssembly : assemblyList) {
                    String assemblyPartNo = orderAssembly.getAssemblyPartNo();
                    if (StringUtils.isNotBlank(assemblyPartNo) && assemblyPartNo.equals(mesPartPtr.getSrcPartNo())){
                        orderAssemblyList = orderAssembly;
                    }
                }
               if (!Objects.isNull(orderAssemblyList)){
                   orderAssemblyList.setAssemblyPartNo(mesPartSap.getPartNo());
                   orderAssemblyList.setAssemblyPartName(mesPartSap.getPartName());
                   orderAssemblyList.setMatchRule(mesPartPtr.getReplacePartSnRule());
                   ptrInfos.append("INFO|").append(mesPartPtr.getId());
                   if (mesPartPtrDetail != null) {
                       ptrInfos.append("PART|").append(mesPartPtrDetail.getId());
                   }
               }
            }

        }
        if (ptrFlag && !fOrderFlag){
            mesWorkOrder.setOrderFlag(MesExtEnumUtil.ORDER_TYPE_IDENTIFICATION.PTR.getValue());
        }else if (ptrFlag){
            mesWorkOrder.setOrderFlag(MesExtEnumUtil.ORDER_TYPE_IDENTIFICATION.F.getValue());
        }else {
            mesWorkOrder.setOrderFlag(MesExtEnumUtil.ORDER_TYPE_IDENTIFICATION.N.getValue());
        }

        //获取车型remark用于目视单打印
        String remark = getCarRemark(partProdGroup.getCustCode(),mesWorkOrder.getCarModelCode(),partProdGroup.getOrganizeCode());
        mesWorkOrder.setDescription(descriptionStr.append(remark).toString());

        /**
         * 工单号修改
         * {ORG}{dynamicRule}{dynamicRule}{dynamicrule}{serialNo}
         * 工厂代码+工单类型 +客户工厂代码+班次+流水号
         */
        GenSerialNoModel genSerialNoModel = new GenSerialNoModel("MES_WORK_ORDER_NO_WU_HAN");
        List<String> resultList = syncFuncService.syncSerialNo(genSerialNoModel.organizeCode(organizeCode), "edi", mesPartProdGroupDetail.getOrganizeCode(), 1).getResultList();
        String serialNo = resultList.get(0).replace("MES_WORK_ORDER_NO_WU_HAN", "");
        String orderNoWuHan = organizeCode + mesWorkOrder.getOrderFlag() + sortInfo.getCustOrderCode() + mesWorkOrder.getShiftCode() + serialNo;
        mesWorkOrder.setWorkOrderNo(orderNoWuHan);

        if (!workOrderPartList.isEmpty()) {
            for (MesWorkOrderPart orderPart : workOrderPartList) {
                orderPart.setOrganizeCode(organizeCode);
                orderPart.setWorkOrderNo(mesWorkOrder.getWorkOrderNo());
                ConvertBean.saveOrUpdate(orderPart, "MES-JOB");
            }
            workOrderPartRao.saveAll(workOrderPartList);
            mesWorkOrder.setProductSeq(productSeq);
            mesWorkOrder.setPartProdGroupName(partProdGroup.getPartProdGroupName());
            ConvertBean.saveOrUpdate(mesWorkOrder, "MES-JOB");
            mesWorkOrderRao.insert(mesWorkOrder);
            /**
             * 插入工单打印队列数据
             */
            mesWorkOrderService.doSaveWorkOrderPrintQueue(mesWorkOrder);
            if (!assemblyList.isEmpty()) {
                for (MesWorkOrderAssembly orderAssembly : assemblyList) {
                    orderAssembly.setPartNo(mesWorkOrder.getPartNo());
                    orderAssembly.setPartName(mesWorkOrder.getPartName());
                    orderAssembly.setWorkOrderNo(mesWorkOrder.getWorkOrderNo());
                    orderAssembly.setAssemblyStatus(MesExtEnumUtil.ASSEMBLY_STATUS.ASSEMBLY_STATUS_30.getValue());
                    orderAssembly.setOrganizeCode(organizeCode);
                    orderAssembly.setProductSn(mesWorkOrder.getWorkOrderNo());
                    orderAssembly.setAreaCode(mesWorkOrder.getAreaCode());
                    ConvertBean.saveOrUpdate(orderAssembly, "MES-JOB");
                }
                workOrderAssemblyRao.saveAll(assemblyList);

                List<String> cellCodeList = assemblyList.stream().map(MesWorkOrderAssembly::getWorkCellCode).distinct().collect(Collectors.toList());
                Map<String, List<MesWorkOrderAssembly>> cellCodeMap = assemblyList.stream().collect(Collectors.groupingBy(MesWorkOrderAssembly::getWorkCellCode));
                for (String cellCode : cellCodeList) {
                    MesQueueOrder mesQueueOrder = new MesQueueOrder();
                    mesQueueOrder.setCarModelCode(mesWorkOrder.getCarModelCode());
                    mesQueueOrder.setWorkOrderNo(mesWorkOrder.getWorkOrderNo());
                    mesQueueOrder.setOrganizeCode(organizeCode);
                    mesQueueOrder.setWorkCellCode(cellCode);
                    mesQueueOrder.setPartProdGroupCode(partProdGroup.getPartProdGroupCode());
                    mesQueueOrder.setPartProdGroupName(partProdGroup.getPartProdGroupName());
                    mesQueueOrder.setProcessSeq(cellCodeMap.get(cellCode).get(0).getProcessSeq());
                    mesQueueOrder.setPartNo(mesWorkOrder.getPartNo());
                    mesQueueOrder.setPartName(mesWorkOrder.getPartName());
                    mesQueueOrder.setWorkOrderNo(mesWorkOrder.getWorkOrderNo());
                    mesQueueOrder.setSerialNumber(mesWorkOrder.getWorkOrderNo());
                    mesQueueOrder.setProductSn(mesWorkOrder.getWorkOrderNo());
                    mesQueueOrder.setCustSn(mesWorkOrder.getCustSn());
                    mesQueueOrder.setStatus(MesExtEnumUtil.QUEUE_ORDER_STATUS.CREATE.getValue());
                    mesQueueOrder.setProductSeq(mesWorkOrder.getProductSeq());
                    mesQueueOrder.setWorkOrderSeq(mesWorkOrder.getWorkOrderSeq());
                    mesQueueOrder.setCustPartNo(mesWorkOrder.getCustPartNo());
                    mesQueueOrder.setCustOrderNo(mesWorkOrder.getCustOrderNo());
                    mesQueueOrder.setWorkCenterCode(mesWorkOrder.getWorkCenterCode());
                    mesQueueOrder.setAreaCode(mesWorkOrder.getAreaCode());
                    mesQueueOrder.setOrderFlag(mesWorkOrder.getOrderFlag());
                    ConvertBean.saveOrUpdate(mesQueueOrder, "MES-JOB");
                    queueOrderRao.insert(mesQueueOrder);
                }

            }

            //生成条码表
            MesProduceSn mesProduceSn = new MesProduceSn();
            mesProduceSn.setProductSn(mesWorkOrder.getWorkOrderNo());
            mesProduceSn.setCustSn(mesWorkOrder.getCustSn());
            mesProduceSn.setWorkOrderNo(mesWorkOrder.getWorkOrderNo());
            mesProduceSn.setOrganizeCode(organizeCode);
            mesProduceSn.setFid(UUID.randomUUID().toString());
            mesProduceSn.setCustCode(mesWorkOrder.getCustCode());
            mesProduceSn.setCustPartNo(mesWorkOrder.getCustPartNo());
            mesProduceSn.setPartNo(mesWorkOrder.getPartNo());
            mesProduceSn.setPartName(mesWorkOrder.getPartName());
            mesProduceSn.setShiftCode(mesWorkOrder.getShiftCode());
            mesProduceSn.setWorkCenterCode(mesWorkOrder.getWorkCenterCode());
            mesProduceSn.setPrintStatus(MesExtEnumUtil.PRINT_STATUS.UNPRINT.getValue());
            mesProduceSn.setQcStatus(MesExtEnumUtil.PRODUCE_QC_STATUS.QUALIFIED.getValue());
            mesProduceSn.setQty(1.0);
            mesProduceSn.setSerialNumber(mesWorkOrder.getWorkOrderNo());
            mesProduceSn.setSnStatus(MesExtEnumUtil.PRODUCE_SN_STATUS.CREATE.getValue());
            mesProduceSn.setSnType(MesExtEnumUtil.PRODUCE_SN_TYPE.STANDARD.getValue());
            mesProduceSn.setFid(UUID.randomUUID().toString());
            mesProduceSn.setAreaCode(mesWorkOrder.getAreaCode());
            ConvertBean.saveOrUpdate(mesProduceSn, "MES-JOB");
            mesProduceSnRao.insert(mesProduceSn);
        }
    }

    private String getCarRemark(String custCode,String carModelCode,String organizeCode){
        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getStringEqualPack(custCode,"custCode",ddlPackBean);
        DdlPreparedPack.getStringEqualPack(carModelCode,"carModelCode",ddlPackBean);
        List<MesCustomerCarModel> carModelList = mesCustomerCarModelRepository.findByHqlTopWhere(ddlPackBean, 1);
        if (carModelList != null && !carModelList.isEmpty()){
            return carModelList.get(0).getRemark();
        }
        return "";
    }
    private List<MesWorkOrderAssembly> getAssemblyList(String workCenterCode, String vehicleNo, List<MesWorkOrderPart> workOrderPartList, MesPartProdGroup mesPartProdGroup) {
        //TM_BAS_WORK_ORDER_ASSEMBLY_SETTING 对应  mes_prod_rule_sort_cfg 排序生产规则

        List<MesWorkOrderAssembly> workOrderAssemblyList = new ArrayList<>();
        List<String> partNoList = workOrderPartList.stream().map(MesWorkOrderPart::getPartNo).collect(Collectors.toList());
        DdlPackBean prodRuleSortPackBean = DdlPackBean.getDdlPackBean(mesPartProdGroup.getOrganizeCode());
//        DdlPreparedPack.getStringEqualPack(mesPartProdGroup.getPartProdGroupCode(),"productGroupCode",prodRuleSortPackBean);
        DdlPreparedPack.getStringEqualPack(workCenterCode, "workCenterCode", prodRuleSortPackBean);
        DdlPreparedPack.getInPackList(partNoList, "assemblyPartNo", prodRuleSortPackBean);
        //查询当前产线的所有排序生产规则--分为当前工单清单的装配件
        List<MesProdRuleSortCfg> sortCfgList = mesProdRuleSortCfgRao.findByHqlWhere(prodRuleSortPackBean);
        List<MesProdRuleSortCfg> sortCfgListFilter = sortCfgList.stream().filter(item -> partNoList.contains(item.getAssemblyPartNo())).collect(Collectors.toList());
        Map<String, List<MesWorkOrderPart>> orderPartMap = workOrderPartList.stream().collect(Collectors.groupingBy(MesWorkOrderPart::getPartNo));
        //遍历排序生产规则
        for (MesProdRuleSortCfg sortCfg : sortCfgListFilter) {
            List<MesWorkOrderPart> orderPartList = orderPartMap.get(sortCfg.getAssemblyPartNo());
            MesWorkOrderAssembly orderAssembly = new MesWorkOrderAssembly();
            MesWorkOrderPart orderPart = orderPartList.get(0);
            //1.逻辑储量程序sortCfg.isLogicNum
            if ((sortCfg.getIsLogicNum() !=null && sortCfg.getIsLogicNum() == CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue()) && (sortCfg.getLogicNum()!=null && sortCfg.getLogicNum() >0 )){
                Double total = orderPartList.stream().map(MesWorkOrderPart::getItemQty).reduce(Double::sum).get();
                if (!total.equals(sortCfg.getLogicNum())){
                    log.info("零件号:{}的数量:{}对应排序规则{}的逻辑数量:{}不一致",orderPart.getPartNo(),total,sortCfg.getWorkCenterCode(),sortCfg.getLogicNum());
                    continue;
                }
            }
            //2.当排序规则的车型不为空时，则需要与参数匹配，如果不符合 则不加这个装配项
            if (StringUtils.isNotBlank(sortCfg.getVehicleNo()) && !sortCfg.getVehicleNo().equals(vehicleNo)){
                continue;
            }
            orderAssembly.setAssemblyPartNo(sortCfg.getAssemblyPartNo());
            orderAssembly.setAssemblyPartName(sortCfg.getAssemblyPartName());
            orderAssembly.setProcessName(sortCfg.getProcessName());
            orderAssembly.setWorkCenterCode(sortCfg.getWorkCenterCode());
            orderAssembly.setDisplayRule(sortCfg.getDisplayRule());
            orderAssembly.setMatchRule(sortCfg.getMatchRule());
            orderAssembly.setMatchType(sortCfg.getMatchType());
            orderAssembly.setPrintType(sortCfg.getPrintType());
            orderAssembly.setProcessCode(sortCfg.getProcessCode());
            orderAssembly.setDisplaySeq(sortCfg.getDisplaySeq());
            orderAssembly.setDisplayValue(sortCfg.getDisplayValue());
            orderAssembly.setProcessSeq(sortCfg.getProcessSeq());
            orderAssembly.setCraftName(sortCfg.getCraftName());
            orderAssembly.setCraftCode(sortCfg.getCraftCode());
            orderAssembly.setRouteSeq(sortCfg.getRouteSeq());
//            orderAssembly.setPartNo(orderPart.getPartNo());
//            orderAssembly.setPartName(orderPart.getPartName());
            orderAssembly.setWorkCellCode(sortCfg.getWorkCellCode());
//            orderAssembly.setOrderPartId(orderPart.getId().toString());
            orderAssembly.setEquipmentCode(sortCfg.getEquipmentCode());
            orderAssembly.setReportType(sortCfg.getReportType());
            if (sortCfg.getReportType() != null && sortCfg.getReportType() == MesExtEnumUtil.MES_REPORT_TYPE.REPORT.getValue()){
                orderAssembly.setReportStatus(MesExtEnumUtil.REPORT_STATUS.REPORT_STATUS_30.getValue());
            }else {
                orderAssembly.setReportStatus(MesExtEnumUtil.REPORT_STATUS.REPORT_STATUS_10.getValue());
            }
            orderAssembly.setReportPartNo(sortCfg.getReportPartNo());
            orderAssembly.setImageFileId(sortCfg.getImageFileId());
            orderAssembly.setCraftCode(sortCfg.getCraftCode());
            orderAssembly.setCraftName(sortCfg.getCraftName());
            orderAssembly.setPartTypeCode(sortCfg.getPartTypeCode());
            orderAssembly.setShowColor(sortCfg.getShowColor());
            orderAssembly.setProductPartNo(sortCfg.getProductPartNo());
            orderAssembly.setPid(sortCfg.getId());
            orderAssembly.setWorkCenterCode(workCenterCode);
            workOrderAssemblyList.add(orderAssembly);
        }

        //车型不为空的逻辑--用于报工
        DdlPackBean reportRuleSortPackBean = DdlPackBean.getDdlPackBean(mesPartProdGroup.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(workCenterCode, "workCenterCode", reportRuleSortPackBean);
        DdlPreparedPack.getStringEqualPack(vehicleNo, "vehicleNo", reportRuleSortPackBean);
        DdlPreparedPack.getNotInPack(partNoList, "assemblyPartNo", reportRuleSortPackBean);
        //查询当前产线的所有排序生产规则--分为当前工单清单的装配件
        List<MesProdRuleSortCfg> prodCfgList = mesProdRuleSortCfgRao.findByHqlWhere(reportRuleSortPackBean);
        for (MesProdRuleSortCfg ruleSortCfg : prodCfgList) {
            MesWorkOrderAssembly orderAssembly = new MesWorkOrderAssembly();
            orderAssembly.setAssemblyPartNo(ruleSortCfg.getAssemblyPartNo());
            orderAssembly.setAssemblyPartName(ruleSortCfg.getAssemblyPartName());
            orderAssembly.setProcessName(ruleSortCfg.getProcessName());
            orderAssembly.setWorkCenterCode(ruleSortCfg.getWorkCenterCode());

            orderAssembly.setDisplayRule(ruleSortCfg.getDisplayRule());
            orderAssembly.setMatchRule(ruleSortCfg.getMatchRule());
            orderAssembly.setMatchType(ruleSortCfg.getMatchType());
            orderAssembly.setPrintType(ruleSortCfg.getPrintType());
            orderAssembly.setProcessCode(ruleSortCfg.getProcessCode());
            orderAssembly.setWorkCellCode(ruleSortCfg.getWorkCellCode());
            orderAssembly.setProcessCode(ruleSortCfg.getProcessCode());
            orderAssembly.setProcessName(ruleSortCfg.getProcessName());
            orderAssembly.setCraftCode(ruleSortCfg.getCraftCode());
            orderAssembly.setRouteSeq(ruleSortCfg.getRouteSeq());
            orderAssembly.setEquipmentCode(ruleSortCfg.getEquipmentCode());
            orderAssembly.setReportType(ruleSortCfg.getReportType());
            if (ruleSortCfg.getReportType() != null && ruleSortCfg.getReportType() == MesExtEnumUtil.MES_REPORT_TYPE.REPORT.getValue()){
                orderAssembly.setReportStatus(MesExtEnumUtil.REPORT_STATUS.REPORT_STATUS_30.getValue());
            }else {
                orderAssembly.setReportStatus(MesExtEnumUtil.REPORT_STATUS.REPORT_STATUS_10.getValue());
            }
            orderAssembly.setReportPartNo(ruleSortCfg.getReportPartNo());
            orderAssembly.setProcessSeq(ruleSortCfg.getProcessSeq());
            orderAssembly.setPid(ruleSortCfg.getId());
            orderAssembly.setImageFileId(ruleSortCfg.getImageFileId());
            orderAssembly.setPartTypeCode(ruleSortCfg.getPartTypeCode());
            orderAssembly.setMaxValue(ruleSortCfg.getMaxValue());
            orderAssembly.setMinValue(ruleSortCfg.getMinValue());
            orderAssembly.setProductPartNo(ruleSortCfg.getProductPartNo());
            workOrderAssemblyList.add(orderAssembly);
        }
        //查询所有共用的零件 且没有零件号，多用于螺钉---查询排序加工规则
        DdlPackBean prodRuleSortPublicPackBean = DdlPackBean.getDdlPackBean(mesPartProdGroup.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(workCenterCode, "workCenterCode", prodRuleSortPublicPackBean);
        DdlPreparedPack.getIsNull("assemblyPartNo", prodRuleSortPublicPackBean);
        List<MesProdRuleSortCfg> ruleSortCfgs = mesProdRuleSortCfgRao.findByHqlWhere(prodRuleSortPublicPackBean);
        for (MesProdRuleSortCfg ruleSortCfg : ruleSortCfgs) {
            MesWorkOrderAssembly orderAssembly = new MesWorkOrderAssembly();
            orderAssembly.setAssemblyPartNo(ruleSortCfg.getAssemblyPartNo());
            orderAssembly.setAssemblyPartName(ruleSortCfg.getAssemblyPartName());
            orderAssembly.setProcessName(ruleSortCfg.getProcessName());
            orderAssembly.setWorkCenterCode(ruleSortCfg.getWorkCenterCode());
            orderAssembly.setDisplayRule(ruleSortCfg.getDisplayRule());
            orderAssembly.setMatchRule(ruleSortCfg.getMatchRule());
            orderAssembly.setMatchType(ruleSortCfg.getMatchType());
            orderAssembly.setPrintType(ruleSortCfg.getPrintType());
            orderAssembly.setProcessCode(ruleSortCfg.getProcessCode());
            orderAssembly.setWorkCellCode(ruleSortCfg.getWorkCellCode());
            orderAssembly.setProcessCode(ruleSortCfg.getProcessCode());
            orderAssembly.setProcessName(ruleSortCfg.getProcessName());
            orderAssembly.setCraftCode(ruleSortCfg.getCraftCode());
            orderAssembly.setRouteSeq(ruleSortCfg.getRouteSeq());
            orderAssembly.setEquipmentCode(ruleSortCfg.getEquipmentCode());
            orderAssembly.setReportType(ruleSortCfg.getReportType());
            if (ruleSortCfg.getReportType() != null && ruleSortCfg.getReportType() == MesExtEnumUtil.MES_REPORT_TYPE.REPORT.getValue()){
                orderAssembly.setReportStatus(MesExtEnumUtil.REPORT_STATUS.REPORT_STATUS_30.getValue());
            }else {
                orderAssembly.setReportStatus(MesExtEnumUtil.REPORT_STATUS.REPORT_STATUS_10.getValue());
            }
            orderAssembly.setReportPartNo(ruleSortCfg.getReportPartNo());
            orderAssembly.setProcessSeq(ruleSortCfg.getProcessSeq());
            orderAssembly.setPid(ruleSortCfg.getId());
            orderAssembly.setImageFileId(ruleSortCfg.getImageFileId());
            orderAssembly.setPartTypeCode(ruleSortCfg.getPartTypeCode());
            orderAssembly.setMaxValue(ruleSortCfg.getMaxValue());
            orderAssembly.setMinValue(ruleSortCfg.getMinValue());
            orderAssembly.setProductPartNo(ruleSortCfg.getProductPartNo());
            workOrderAssemblyList.add(orderAssembly);
        }
        return workOrderAssemblyList;
    }

    private MesProdRuleSortCfg getSortRuleCfg (String organizeCode,String workCenterCode,String partNo ){
        DdlPackBean prodRuleSortPackBean = DdlPackBean.getDdlPackBean(organizeCode);
//        DdlPreparedPack.getStringEqualPack(mesPartProdGroup.getPartProdGroupCode(),"productGroupCode",prodRuleSortPackBean);
        DdlPreparedPack.getStringEqualPack(workCenterCode, "workCenterCode", prodRuleSortPackBean);
        DdlPreparedPack.getStringEqualPack(partNo, "assemblyPartNo", prodRuleSortPackBean);
        //查询当前产线的所有排序生产规则--分为当前工单清单的装配件
        List<MesProdRuleSortCfg> sortCfgList = mesProdRuleSortCfgRao.findByHqlTopWhere(prodRuleSortPackBean,1);

        if (sortCfgList != null && !sortCfgList.isEmpty()){
            return sortCfgList.get(0);
        }
        return null;
    }
    private List<MesWorkOrderPart> dismantleBom(String partNo, double qty, String productTime, String bomCode, String organizeCode, String workCellCode, String workOrderNo) {
        List<MesWorkOrderPart> orderPartList = new ArrayList<>();

        //1.根据bomCode,partNo productTime organizeCode 查询bom清单 bomList
        List<MesBom> bomList = getPlatBom(partNo, productTime, bomCode, organizeCode);
        //2.根据bomList中的subPartNo 查询零件清单 partList
        List<String> partNoList = bomList.stream().map(MesBom::getItemPartNo).distinct().collect(Collectors.toList());

        if (partNoList.isEmpty()) {
            return new ArrayList<>();
        }
        DdlPackBean partPackBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getInPackList(partNoList, "partNo", partPackBean);
        List<MesPartSap> partSapList = mesPartSapRao.findByHqlWhere(partPackBean);
        Map<String, MesPartSap> partMap = partSapList.stream().collect(Collectors.toMap(MesPartSap::getPartNo, Function.identity()));


        for (MesBom bom : bomList) {
            MesWorkOrderPart orderPart = new MesWorkOrderPart();
            MesPartSap partSap = partMap.get(bom.getItemPartNo());
            if (partSap == null) {
                log.info("工厂:{},零件号:{}未查到零件信息", organizeCode, bom.getItemPartNo());
                return new ArrayList<>();
            }
            List<MesWorkOrderPart> orderParts = orderPartList.stream().filter(item -> item.getPartNo().equals(partSap.getPartNo())).collect(Collectors.toList());
            if (!orderParts.isEmpty()) {
                MesWorkOrderPart tempPart = orderParts.get(0);
                tempPart.setItemQty(tempPart.getItemQty() + bom.getItemQty() * qty);
                continue;
            }
            orderPart.setPartName(partSap.getPartName());
            orderPart.setPartNo(partSap.getPartNo());
            orderPart.setItemQty(bom.getItemQty() * qty);
            orderPart.setItemUnit(partSap.getUnit());
            orderPart.setSourceId(bom.getId());
            orderPart.setShippingPartNo(partSap.getPartNo());
            orderPart.setWorkOrderNo(workOrderNo);
            orderPart.setWorkCellCode(workCellCode);
            orderPartList.add(orderPart);
        }
        //3.遍历bomList
        return orderPartList;
    }

    private List<MesBom> getPlatBom(String partNo, String effectiveTime, String bomCode, String organizeCode) {
        if (bomCode == null) {
            //首先根据虚结构的零件号查询出最新的一条bom信息作为bomCode
            DdlPackBean bomCodePackBean = DdlPackBean.getDdlPackBean(organizeCode);
            DdlPreparedPack.getStringEqualPack(partNo, "partNo", bomCodePackBean);
            DdlPreparedPack.getTimeBetweenCol(effectiveTime, "effStartTime", "effEndTime", bomCodePackBean, false);
            DdlPreparedPack.getOrderBy("effStartTime", CommonEnumUtil.ASC_OR_DESC.DESC.getValue(), bomCodePackBean);
            List<MesBom> bomList = bomRao.findByHqlTopWhere(bomCodePackBean, 1);
            if (bomList == null || bomList.isEmpty()) {
                return new ArrayList<>();
            }
            bomCode = bomList.get(0).getBomCode();
        }
        List<MesBom> bomResultList = new ArrayList<>();
        DdlPackBean bomPackBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getStringEqualPack(bomCode, "bomCode", bomPackBean);
        DdlPreparedPack.getStringEqualPack(partNo, "partNo", bomPackBean);
        DdlPreparedPack.getStringSmallerPack(effectiveTime, "effStartTime", bomPackBean);
        DdlPreparedPack.getStringBiggerPack(effectiveTime, "effEndTime", bomPackBean);
        List<MesBom> bomList = bomRao.findByHqlWhere(bomPackBean);

        for (MesBom bom : bomList) {
            if (bom.getPartType() != null && bom.getPartType().equalsIgnoreCase("X")) {
                getPlatBom(bom.getItemPartNo(), effectiveTime, null, organizeCode);
            } else {
                bom.setBomCode(bom.getBomCode().toLowerCase());
                bomResultList.add(bom);
            }

        }

        return bomResultList;
    }

    private MesPartSap getMesPart(String productPartNo, String organizeCode) {
        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getStringEqualPack(productPartNo, "partNo", ddlPackBean);
        List<MesPartSap> parts = mesPartSapRao.findByHqlWhere(ddlPackBean);
        return parts.isEmpty() ? null : parts.get(0);
    }

    private MesProductVersion getProdVersion(String organizeCode, String partNo) {
        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getStringEqualPack(partNo, "partNo", ddlPackBean);
        DdlPreparedPack.getOrderBy("createDatetime",CommonEnumUtil.ASC_OR_DESC.DESC.getValue(), ddlPackBean);
        List<MesProductVersion> mesProductVersionList = mesProductVersionRao.findByHqlWhere(ddlPackBean);
        return mesProductVersionList.isEmpty() ? null : mesProductVersionList.get(0);
    }


    private List<MesPartPtr> getPartPtrList(String organizeCode,String workCenterCode, List<String> partNoList) {
        /**
         * 2024/11/16
         * PTR生效
         * 1.有开始时间和结束时间 和 需要数量，达到其中一个就结束
         * 2.有开始时间无结束时间 和 需要数量 达到需要的数量结束
         * 3.有开始时间无结束时间，无结束数量 一直替换
         */
        DdlPackBean partPtrPackBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getStringEqualPack(workCenterCode, "workCenterCode", partPtrPackBean);
        DdlPreparedPack.getInPackList(partNoList, "srcPartNo", partPtrPackBean);
        DdlPreparedPack.getStringSmallerPack(TimeTool.getNowTime("yyyy-MM-dd HH:mm:ss"), "startTime", partPtrPackBean);
        List<MesPartPtr> partPtrList = partPtrRao.findByHqlWhere(partPtrPackBean);
        /**
         * 有开始时间 和 无结束时间  和 需要的数量，只有达到数量才结束
         */

        DdlPackBean part1PtrPackBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getStringEqualPack(workCenterCode, "workCenterCode", part1PtrPackBean);
        DdlPreparedPack.getInPackList(partNoList, "srcPartNo", part1PtrPackBean);
        DdlPreparedPack.getStringSmallerPack(TimeTool.getNowTime("yyyy-MM-dd HH:mm:ss"), "startTime", part1PtrPackBean);
        DdlPreparedPack.getIsNotNull("prtVehicleCount", part1PtrPackBean);
        DdlPreparedPack.getIsNull("endTime", part1PtrPackBean);
        List<MesPartPtr> partPtrList1 = partPtrRao.findByHqlWhere(part1PtrPackBean);
        partPtrList.addAll(partPtrList1);

        partPtrList = partPtrList.stream().filter(item -> item.getPrtVehicleCount() != null && item.getPrtVehicleCount() > 0)
                .filter(item-> item.getPrtVehicleCount() > item.getPrtVehicleActualCount())
                .collect(Collectors.toList());
        /**
         * 有开始时间 和 无结束时间 和 无需要的数量
         */

        DdlPackBean part2PtrPackBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getStringEqualPack(workCenterCode, "workCenterCode", part2PtrPackBean);
        DdlPreparedPack.getInPackList(partNoList, "srcPartNo", part2PtrPackBean);
        DdlPreparedPack.getStringSmallerPack(TimeTool.getNowTime("yyyy-MM-dd HH:mm:ss"), "startTime", part2PtrPackBean);
        DdlPreparedPack.getIsNull("prtVehicleCount", part2PtrPackBean);
        DdlPreparedPack.getIsNull("endTime", part2PtrPackBean);
        List<MesPartPtr> partPtrList2 = partPtrRao.findByHqlWhere(part2PtrPackBean);
        partPtrList.addAll(partPtrList2);
        return partPtrList;
    }
    //根据排序信息PTR零件号 获取 ptr信息
    private List<MesPartPtr> getCustPartPtrList(List<String> ptrInfoList,String organizeCode){
        DdlPackBean partPtrPackBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getInPackList(ptrInfoList, "seqInfoPrtPart", partPtrPackBean);
        return partPtrRao.findByHqlWhere(partPtrPackBean);

    }
}
