package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.busi;

import cn.estsh.i3plus.ext.mes.api.base.IMesConfigService;
import cn.estsh.i3plus.ext.mes.api.base.IMesEquipmentSpotCheckService;
import cn.estsh.i3plus.ext.mes.api.busi.IMesSpotCheckOrderPartService;
import cn.estsh.i3plus.ext.mes.api.busi.IMesSpotCheckOrderResultService;
import cn.estsh.i3plus.ext.mes.api.busi.IMesSpotCheckOrderService;
import cn.estsh.i3plus.ext.mes.apiservice.config.AuthUtilExt;
import cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base.BaseMesService;
import cn.estsh.i3plus.ext.mes.pojo.constant.MesCommonConstant;
import cn.estsh.i3plus.icloud.core.sdk.ICoreUtilCloud;
import cn.estsh.i3plus.mes.api.iservice.base.IStateMachineRouteDesignService;
import cn.estsh.i3plus.mes.api.iservice.busi.ISyncFuncService;
import cn.estsh.i3plus.mes.apiservice.util.DateUtil;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.platform.common.tool.TimeTool;
import cn.estsh.i3plus.platform.common.util.WmsConstWords;
import cn.estsh.i3plus.pojo.base.bean.BaseResultBean;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.*;
import cn.estsh.i3plus.pojo.mes.model.GenSerialNoModel;
import cn.estsh.i3plus.pojo.mes.repository.MesEquipmentSpotCheckDetailRepository;
import cn.estsh.i3plus.pojo.mes.repository.MesEquipmentSpotCheckPartRepository;
import cn.estsh.i3plus.pojo.mes.repository.MesSpotCheckOrderPartRepository;
import cn.estsh.i3plus.pojo.mes.repository.MesSpotCheckOrderResultRepository;
import cn.estsh.i3plus.pojo.mes.util.DateUtilExt;
import cn.estsh.i3plus.pojo.mes.util.MesExtEnumUtil;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.*;
import java.util.stream.Collectors;

@Service
@Slf4j
public class MesSpotCheckOrderService extends BaseMesService<MesSpotCheckOrder> implements IMesSpotCheckOrderService {


    @Autowired
    private IMesEquipmentSpotCheckService equipmentSpotCheckService;
    @Autowired
    private IMesSpotCheckOrderResultService spotCheckOrderResultService;
    @Autowired
    private MesSpotCheckOrderResultRepository spotCheckOrderResultRepository;
    @Autowired
    private IMesSpotCheckOrderPartService spotCheckOrderPartService;

    @Autowired
    private MesSpotCheckOrderPartRepository spotCheckOrderPartRepository;

    @Autowired
    private MesEquipmentSpotCheckDetailRepository equipmentSpotCheckDetailRepository;

    @Autowired
    private MesEquipmentSpotCheckPartRepository equipmentSpotCheckPartRepository;

    @Autowired
    private ISyncFuncService syncFuncService;
    @Autowired
    private ICoreUtilCloud sysOrderNoRuleCloud;

    @Autowired
    private IStateMachineRouteDesignService stateMachineRouteDesignService;
    @Autowired
    private IMesConfigService mesConfigService;
    @Override
    public MesSpotCheckOrder insert(MesSpotCheckOrder bean) {
        onInsertBean(bean);

        bean.setTaskResource(MesCommonConstant.SPOT_CHECK_ORDER_TASK_RESOURCE);
        bean.setStatus(MesExtEnumUtil.SPOT_CHECK_ORDER_STATUS.CREATE.getValue());
        //查询点检方案  点检单点检方案  spotCheckId-id关联
        MesEquipmentSpotCheck equipmentSpotCheck = equipmentSpotCheckService.get(bean.getSpotCheckId());

        if (StringUtils.isEmpty(equipmentSpotCheck)) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("【%s】点检方案不存在，请检查数据", bean.getSpotCheckId())
                    .build();
        }

        if (equipmentSpotCheck.getTriggerMode() == MesExtEnumUtil.EQUIPMENT_CHECK_INSPECTION_TRIGGER_METHOD.TIME_TRIGGERED.getValue()) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("【%s】点检方案触发方式为【%s】，请勿手动创建", bean.getSpotCheckId(), MesExtEnumUtil.EQUIPMENT_CHECK_INSPECTION_TRIGGER_METHOD.TIME_TRIGGERED.getDescription())
                    .build();
        }

        //查询点检明细 主表明细表 spotCheckId-pid关联
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(bean.getOrganizeCode());
        DdlPreparedPack.getNumEqualPack(bean.getSpotCheckId(), "pid", packBean);
        List<MesEquipmentSpotCheckDetail> detailList = equipmentSpotCheckDetailRepository.findByHqlWhere(packBean);

        if (CollectionUtils.isEmpty(detailList)) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("【%s】点检方案明细不存在，请检查数据", bean.getSpotCheckId())
                    .build();
        }

        //查询点检零件 主表零件表 spotCheckId-pid关联
        List<MesEquipmentSpotCheckPart> equipmentSpotCheckPartList = equipmentSpotCheckPartRepository.findByHqlWhere(packBean);

        if (CollectionUtils.isEmpty(equipmentSpotCheckPartList) &&
                equipmentSpotCheck.getSpotCheckOrderType() == MesExtEnumUtil.SPOT_CHECK_ORDER_TYPE.FIRST_PRODUCT_INSPECTION.getValue()) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("【%s】点检方案类型为【%s】配置零件号为空，请检查数据", bean.getSpotCheckId(), MesExtEnumUtil.SPOT_CHECK_ORDER_TYPE.FIRST_PRODUCT_INSPECTION.getDescription())
                    .build();
        }

        //生成单据号
//        GenSerialNoModel genSerialNoModel = new GenSerialNoModel(MesCommonConstant.SPOT_CHECK_ORDER_NO_RULE);
//        List<String> resultList = syncFuncService.syncSerialNo(genSerialNoModel, bean.getCreateUser(), bean.getOrganizeCode(), 1).getResultList();

        String orderNo="";
        BaseResultBean<LinkedHashMap> orderResutlBean = sysOrderNoRuleCloud.getOrderNo(MesCommonConstant.SPOT_CHECK_ORDER_NO_RULE);
        if (orderResutlBean != null && orderResutlBean.isSuccess()) {
            orderNo = orderResutlBean.getResultObject().get(WmsConstWords.CORE_ORDER_NO).toString();
        } else {
            orderNo = TimeTool.getDateTime().replaceAll("-", "") + String.valueOf((int) (Math.random() * 9000) + 1000);
        }
        //保存单据
        BeanUtils.copyProperties(equipmentSpotCheck, bean, "id");

        bean.setSpotCheckOrderNo(orderNo);
        bean.setSystemSyncStatus(CommonEnumUtil.FALSE);
        bean.setSystemSyncDatetime("");
        ConvertBean.serviceModelInitialize(bean, AuthUtil.getSessionUser().getUserName());
        bean = baseRDao.insert(bean);

        //保存单据明细
        List<MesSpotCheckOrderResult> spotCheckOrderResults = new ArrayList<>();
        MesSpotCheckOrder finalBean = bean;
        detailList.forEach(k -> {
            MesSpotCheckOrderResult result = new MesSpotCheckOrderResult();
            BeanUtils.copyProperties(k, result, "id");
            result.setSpotCheckOrderNo(finalBean.getSpotCheckOrderNo());
            result.setPid(finalBean.getId());
            result.setWorkCenterCode(finalBean.getWorkCenterCode());
            result.setWorkCellCode(finalBean.getWorkCellCode());
            result.setSpotCheckItemId(k.getId());
            result.setTaskItemName(k.getSpotCheckItemName());
            result.setSpotCheckStandardValue(k.getStandardValue());
            result.setSystemSyncStatus(CommonEnumUtil.FALSE);
            result.setSystemSyncDatetime("");
            ConvertBean.serviceModelInitialize(result, AuthUtil.getSessionUser().getUserName());
//            ConvertBean.serviceModelInitialize(result, "LML");
            spotCheckOrderResults.add(result);
        });

        //保存单据零件
        if (!CollectionUtils.isEmpty(equipmentSpotCheckPartList)) {

            List<MesSpotCheckOrderPart> spotCheckOrderPartList = new ArrayList<>();
            equipmentSpotCheckPartList.forEach(k -> {
                MesSpotCheckOrderPart spotCheckOrderPart = new MesSpotCheckOrderPart();
                spotCheckOrderPart.setSpotCheckOrderId(finalBean.getId());
                BeanUtils.copyProperties(k, spotCheckOrderPart, "id");
                spotCheckOrderPart.setSystemSyncStatus(CommonEnumUtil.FALSE);
                spotCheckOrderPart.setSystemSyncDatetime("");
                ConvertBean.serviceModelInitialize(spotCheckOrderPart, AuthUtil.getSessionUser().getUserName());
//                ConvertBean.serviceModelInitialize(spotCheckOrderPart, "LML");
                spotCheckOrderPartList.add(spotCheckOrderPart);
            });
            spotCheckOrderPartRepository.saveAll(spotCheckOrderPartList);
        }

        spotCheckOrderResultRepository.saveAll(spotCheckOrderResults);

        return bean;
    }

    @Override
    public MesSpotCheckOrder update(MesSpotCheckOrder bean) {
        onUpdateBean(bean);
        //获取点检单
        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(bean.getOrganizeCode());
        DdlPreparedPack.getNumEqualPack(bean.getId(), "id", ddlPackBean);
        MesSpotCheckOrder originBean = baseRDao.getByProperty(ddlPackBean);

        if (StringUtils.isEmpty(originBean)) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("【%s】点检单不存在，请检查数据", bean.getId())
                    .build();
        }

        if (originBean.getStatus() != MesExtEnumUtil.SPOT_CHECK_ORDER_STATUS.CREATE.getValue()) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("【%s】点检单状态为【%s】不允许修改，请检查数据", bean.getSpotCheckId(), MesExtEnumUtil.SPOT_CHECK_ORDER_STATUS.valueOfDescription(originBean.getStatus()))
                    .build();
        }

        //获取点检单明细
        DdlPackBean orderResultPackBean = DdlPackBean.getDdlPackBean(bean.getOrganizeCode());
        DdlPreparedPack.getNumEqualPack(bean.getId(), "pid", orderResultPackBean);
        List<MesSpotCheckOrderResult> oldOrderResultList = spotCheckOrderResultRepository.findByHqlWhere(orderResultPackBean);

        //软删点检单明细
        oldOrderResultList.forEach(k -> {
            k.setSystemSyncStatus(CommonEnumUtil.FALSE);
            k.setSystemSyncDatetime("");
            k.setIsDeleted(CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue());
            ConvertBean.serviceModelUpdate(k, AuthUtil.getSessionUser().getUserName());
        });
        spotCheckOrderResultRepository.saveAll(oldOrderResultList);

        //获取点检单零件号
        DdlPackBean orderPartPackBean = DdlPackBean.getDdlPackBean(bean.getOrganizeCode());
        DdlPreparedPack.getNumEqualPack(bean.getId(), "spotCheckOrderId", orderPartPackBean);
        List<MesSpotCheckOrderPart> oldOrderPartList = spotCheckOrderPartRepository.findByHqlWhere(orderPartPackBean);

        //软删点检单零件号及条码
        oldOrderPartList.forEach(k -> {
            k.setIsDeleted(CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue());
            k.setSystemSyncStatus(CommonEnumUtil.FALSE);
            k.setSystemSyncDatetime("");
            ConvertBean.serviceModelUpdate(k, AuthUtil.getSessionUser().getUserName());
        });
        spotCheckOrderPartRepository.saveAll(oldOrderPartList);

        //获取点检方案 点检单点检方案  spotCheckId-id关联
        MesEquipmentSpotCheck equipmentSpotCheck = equipmentSpotCheckService.get(bean.getSpotCheckId());

        if (StringUtils.isEmpty(equipmentSpotCheck)) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("【%s】点检方案不存在，请检查数据", bean.getSpotCheckId())
                    .build();
        }

        if (equipmentSpotCheck.getTriggerMode() == MesExtEnumUtil.EQUIPMENT_SPOT_CHECK_TRIGGER_MODE.TIME.getValue()) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("【%s】点检方案触发方式为【%s】，请勿手动创建", bean.getSpotCheckId(), MesExtEnumUtil.EQUIPMENT_SPOT_CHECK_TRIGGER_MODE.TIME.getDescription())
                    .build();
        }

        //获取点检方案明细 主表明细表 spotCheckId-pid关联
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(bean.getOrganizeCode());
        DdlPreparedPack.getNumEqualPack(bean.getSpotCheckId(), "pid", packBean);
        List<MesEquipmentSpotCheckDetail> detailList = equipmentSpotCheckDetailRepository.findByHqlWhere(packBean);

        if (CollectionUtils.isEmpty(detailList)) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("【%s】点检方案明细不存在，请检查数据", bean.getSpotCheckId())
                    .build();
        }

        //获取点检方案零件 主表零件表 spotCheckId-pid关联
        List<MesEquipmentSpotCheckPart> equipmentSpotCheckPartList = equipmentSpotCheckPartRepository.findByHqlWhere(packBean);

        if (CollectionUtils.isEmpty(equipmentSpotCheckPartList) &&
                equipmentSpotCheck.getSpotCheckOrderType() == MesExtEnumUtil.SPOT_CHECK_ORDER_TYPE.FIRST_PRODUCT_INSPECTION.getValue()) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("【%s】点检方案类型为【%s】配置零件号为空，请检查数据", bean.getSpotCheckId(), MesExtEnumUtil.SPOT_CHECK_ORDER_TYPE.FIRST_PRODUCT_INSPECTION.getDescription())
                    .build();
        }

        List<MesSpotCheckOrderResult> spotCheckOrderResults = new ArrayList<>();
        MesSpotCheckOrder finalBean = bean;
        detailList.forEach(k -> {
            MesSpotCheckOrderResult result = new MesSpotCheckOrderResult();
            BeanUtils.copyProperties(k, result, "id");
            result.setPid(finalBean.getId());
            result.setSystemSyncStatus(CommonEnumUtil.FALSE);
            result.setSystemSyncDatetime("");
            ConvertBean.serviceModelInitialize(result, AuthUtil.getSessionUser().getUserName());
            spotCheckOrderResults.add(result);
        });

        if (!CollectionUtils.isEmpty(equipmentSpotCheckPartList)) {

            List<MesSpotCheckOrderPart> spotCheckOrderPartList = new ArrayList<>();
            equipmentSpotCheckPartList.forEach(k -> {
                MesSpotCheckOrderPart spotCheckOrderPart = new MesSpotCheckOrderPart();
                spotCheckOrderPart.setSpotCheckOrderId(finalBean.getId());
                BeanUtils.copyProperties(k, spotCheckOrderPart, "id");
                spotCheckOrderPart.setSystemSyncStatus(CommonEnumUtil.FALSE);
                spotCheckOrderPart.setSystemSyncDatetime("");
                ConvertBean.serviceModelInitialize(spotCheckOrderPart, AuthUtil.getSessionUser().getUserName());
                spotCheckOrderPartList.add(spotCheckOrderPart);
            });
            spotCheckOrderPartService.insertBatch(spotCheckOrderPartList);
        }


        spotCheckOrderResultService.insertBatch(spotCheckOrderResults);
        bean.setSystemSyncStatus(CommonEnumUtil.FALSE);
        bean.setSystemSyncDatetime("");
        ConvertBean.serviceModelUpdate(bean, AuthUtil.getSessionUser().getUserName());
        baseRDao.update(bean);
        return originBean;
    }

    @Override
    public void deleteWeaklyByIds(Long[] ids, String userName) {
        String organizeCode = AuthUtilExt.getOrganizeCode();
        for (Long id : ids) {
            DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(organizeCode);
            DdlPreparedPack.getNumEqualPack(id, "id", ddlPackBean);
            MesSpotCheckOrder bean = baseRDao.getByProperty(ddlPackBean);
            if (StringUtils.isEmpty(bean)) continue;

            if (bean.getStatus() == MesExtEnumUtil.SPOT_CHECK_ORDER_STATUS.COMPLETE.getValue() && Objects.equals(bean.getSpotCheckOrderResult(),
                    MesExtEnumUtil.SPOT_CHECK_ORDER_RESULT_TYPE.OK.getValue())) {
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                        .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                        .setErrorDetail("【%s】点检单状态为【%s】,且点检结果为OK,不允许修改，请检查数据", bean.getSpotCheckId(), MesExtEnumUtil.SPOT_CHECK_ORDER_STATUS.valueOfDescription(bean.getStatus()))
                        .build();
            }
            bean.setSystemSyncStatus(CommonEnumUtil.FALSE);
            bean.setIsDeleted(CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue());
            ConvertBean.serviceModelUpdate(bean, AuthUtil.getSessionUser().getUserName());
            //获取点检方案明细
            DdlPackBean packBean = DdlPackBean.getDdlPackBean(bean.getOrganizeCode());
            DdlPreparedPack.getNumEqualPack(bean.getId(), "pid", packBean);
            List<MesSpotCheckOrderResult> detailList = spotCheckOrderResultRepository.findByHqlWhere(packBean);
            detailList.forEach(k -> {
                k.setIsDeleted(CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue());
                k.setSystemSyncStatus(CommonEnumUtil.FALSE);
                k.setSystemSyncDatetime("");
                ConvertBean.serviceModelUpdate(k, AuthUtil.getSessionUser().getUserName());
            });
            spotCheckOrderResultRepository.saveAll(detailList);

            //获取点检方案零件 主表零件表 spotCheckId-pid关联
            //获取点检单零件号
            DdlPackBean orderPartPackBean = DdlPackBean.getDdlPackBean(bean.getOrganizeCode());
            DdlPreparedPack.getNumEqualPack(bean.getId(), "spotCheckOrderId", orderPartPackBean);
            List<MesSpotCheckOrderPart> oldOrderPartList = spotCheckOrderPartRepository.findByHqlWhere(orderPartPackBean);

            if (CollectionUtils.isEmpty(oldOrderPartList)) {
                oldOrderPartList.forEach(k -> {
                    k.setIsDeleted(CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue());
                    k.setSystemSyncStatus(CommonEnumUtil.FALSE);
                    k.setSystemSyncDatetime("");
                    ConvertBean.serviceModelUpdate(k, AuthUtil.getSessionUser().getUserName());
                });
                spotCheckOrderPartRepository.saveAll(oldOrderPartList);
            }

        }
    }

    protected void onInsertBean(MesSpotCheckOrder bean) {

        // 数据校验
        ValidatorBean.checkNotNull(bean.getSpotCheckId(), "点检方案不能为空");

        DdlPackBean packBean = DdlPackBean.getDdlPackBean(bean.getOrganizeCode());
        DdlPreparedPack.getNumEqualPack(bean.getSpotCheckId(), "spotCheckId", packBean);
        DdlPreparedPack.getNumEqualPack(MesExtEnumUtil.SPOT_CHECK_ORDER_STATUS.CREATE.getValue(), "status", packBean);
        boolean flg = baseRDao.isExitByHql(packBean);
        if (flg) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("存在相同配置状态为【%s】的点检单 id：【%s】，请检查数据", MesExtEnumUtil.SPOT_CHECK_ORDER_STATUS.CREATE.getDescription(), bean.getSpotCheckId())
                    .build();
        }
    }

    protected void onUpdateBean(MesSpotCheckOrder bean) {

        // 数据校验
        ValidatorBean.checkNotNull(bean.getSpotCheckId(), "点检方案不能为空");

        DdlPackBean packBean = DdlPackBean.getDdlPackBean(bean.getOrganizeCode());
        DdlPreparedPack.getNumEqualPack(bean.getSpotCheckId(), "spotCheckId", packBean);
        DdlPreparedPack.getNumEqualPack(MesExtEnumUtil.SPOT_CHECK_ORDER_STATUS.CREATE.getValue(), "status", packBean);
        boolean flg = baseRDao.isExitByHql(packBean);
        if (flg) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("存在相同配置状态为【%s】的点检单 id：【%s】，请检查数据", MesExtEnumUtil.SPOT_CHECK_ORDER_STATUS.CREATE.getDescription(), bean.getSpotCheckId())
                    .build();
        }
    }


    protected void setPackQueryBean(MesSpotCheckOrder bean, DdlPackBean packBean) {
        DdlPreparedPack.getStringLikerPack(bean.getSpotCheckOrderNo(),"spotCheckOrderNo",packBean);
        DdlPreparedPack.getStringLikerPack(bean.getWorkCenterCode(),"workCenterCode",packBean);
        DdlPreparedPack.getStringLikerPack(bean.getWorkCellCode(),"workCellCode",packBean);
        DdlPreparedPack.getStringLikerPack(bean.getSpotCheckCode(),"spotCheckCode",packBean);
        DdlPreparedPack.getStringLikerPack(bean.getSpotCheckName(),"spotCheckName",packBean);
        DdlPreparedPack.getStringLikerPack(bean.getEquipCode(),"equipCode",packBean);
        DdlPreparedPack.getNumEqualPack(bean.getStatus(),"status",packBean);
        DdlPreparedPack.getNumEqualPack(bean.getSpotCheckOrderResult(),"spotCheckOrderResult",packBean);
        DdlPreparedPack.getStringLikerPack(bean.getEquipName(),"equipName",packBean);
        DdlPreparedPack.getStringBiggerPack(bean.getModifyDateTimeStart(), "spotCheckTime", packBean);
        DdlPreparedPack.getStringSmallerPack(bean.getModifyDateTimeEnd(), "spotCheckTime", packBean);
        DdlPreparedPack.getOrderByPack(new Object[]{CommonEnumUtil.ASC_OR_DESC.DESC.getValue()}, new String[]{"createDatetime"}, packBean);
    }

    @Override
    public void insertMesSpotCheckOrder(List<MesEquipmentSpotCheck> mesEquipmentSpotCheckList,String userName) {
        MesEquipmentSpotCheck next = mesEquipmentSpotCheckList.iterator().next();
        List<Long> spotCheckIdList = mesEquipmentSpotCheckList.stream().map(MesEquipmentSpotCheck::getId).collect(Collectors.toList());
        //查询未处理的单据
        Map<Long, MesSpotCheckOrder> spotCheckOrderMap = getMesSpotCheckOrderMap(next.getOrganizeCode(), spotCheckIdList);
        //查询明细数据
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(next.getOrganizeCode());
        DdlPreparedPack.getInPackList(spotCheckIdList,"pid", packBean);
        List<MesEquipmentSpotCheckDetail> detailList = equipmentSpotCheckDetailRepository.findByHqlWhere(packBean);
        Map<Long,List<MesEquipmentSpotCheckDetail>> detailMap = null;
        if (!CollectionUtils.isEmpty(detailList)) {
            detailMap = detailList.stream().collect(Collectors.groupingBy(MesEquipmentSpotCheckDetail::getPid));
        }
        //查询点检零件
        List<MesEquipmentSpotCheckPart> equipmentSpotCheckPartList = equipmentSpotCheckPartRepository.findByHqlWhere(packBean);
        Map<Long,List<MesEquipmentSpotCheckPart>> checkPartListMap  = null;
        if (!CollectionUtils.isEmpty(equipmentSpotCheckPartList)) {
            checkPartListMap = equipmentSpotCheckPartList.stream().collect(Collectors.groupingBy(MesEquipmentSpotCheckPart::getPid));
        }
        //属性校验
        List<MesEquipmentSpotCheck> mesEquipmentSpotChecks = getCheckList(mesEquipmentSpotCheckList, spotCheckOrderMap, detailMap, checkPartListMap);
        //保存单据
        if(mesEquipmentSpotChecks.size() > 0){
            saveOrder(userName, next, detailMap, checkPartListMap, mesEquipmentSpotChecks);
        }
    }

    private void saveOrder(String userName, MesEquipmentSpotCheck next, Map<Long, List<MesEquipmentSpotCheckDetail>> detailMap, Map<Long, List<MesEquipmentSpotCheckPart>> checkPartListMap, List<MesEquipmentSpotCheck> mesEquipmentSpotChecks) {
        //生成单号
        GenSerialNoModel genSerialNoModel = new GenSerialNoModel(MesCommonConstant.SPOT_CHECK_ORDER_NO_RULE);
        List<String> resultList = syncFuncService.syncSerialNo(genSerialNoModel, userName, next.getOrganizeCode(), mesEquipmentSpotChecks.size()).getResultList();
        String orderNo;
        int count = 0 ;
        //单据生成
        for (MesEquipmentSpotCheck spotCheck : mesEquipmentSpotChecks) {
            orderNo = resultList.get(count);
            MesSpotCheckOrder mesSpotCheckOrder = new MesSpotCheckOrder();
            BeanUtils.copyProperties(spotCheck, mesSpotCheckOrder, "id");
            mesSpotCheckOrder.setTaskResource(MesCommonConstant.SPOT_CHECK_ORDER_TASK_RESOURCE);
            mesSpotCheckOrder.setStatus(MesExtEnumUtil.SPOT_CHECK_ORDER_STATUS.CREATE.getValue());
            mesSpotCheckOrder.setSpotCheckId(spotCheck.getId());
            mesSpotCheckOrder.setSpotCheckOrderNo(orderNo);
            mesSpotCheckOrder.setSystemSyncStatus(CommonEnumUtil.FALSE);
            mesSpotCheckOrder.setSystemSyncDatetime("");
            ConvertBean.serviceModelInitialize(mesSpotCheckOrder, userName);
            mesSpotCheckOrder = baseRDao.insert(mesSpotCheckOrder);
            //保存单据明细
            saveMesSpotCheckOrderDetails(userName, detailMap, spotCheck.getId(), mesSpotCheckOrder);
            //保存单据零件
            saveMesEquipmentSpotCheckPartList(userName, checkPartListMap, spotCheck.getId(), mesSpotCheckOrder.getId());
            spotCheck.setSystemSyncStatus(CommonEnumUtil.FALSE);
            spotCheck.setSystemSyncDatetime("");
            ConvertBean.serviceModelUpdate(spotCheck, userName);
            baseRDao.update(spotCheck);
            count++;
        }
    }

    private List<MesEquipmentSpotCheck> getCheckList(List<MesEquipmentSpotCheck> mesEquipmentSpotCheckList, Map<Long, MesSpotCheckOrder> spotCheckOrderMap, Map<Long, List<MesEquipmentSpotCheckDetail>> detailMap, Map<Long, List<MesEquipmentSpotCheckPart>> checkPartListMap) {
        List<MesEquipmentSpotCheck> mesEquipmentSpotChecks = new ArrayList<>();
        for (MesEquipmentSpotCheck spotCheck : mesEquipmentSpotCheckList) {
            log.info("开始处理点检项目代码:{}",spotCheck.getSpotCheckCode());
            //校验是否存在未处理的单据
            if(!Objects.isNull(spotCheckOrderMap) && spotCheckOrderMap.containsKey(spotCheck.getId())){
                log.info("存在相同配置状态为{}的点检单 id：{}，请检查数据",MesExtEnumUtil.SPOT_CHECK_ORDER_STATUS.CREATE.getDescription(), spotCheck.getId());
                continue;
            }
            //时间触发方式
            if (checkTime(spotCheck)) continue;
            //明细是否存在
            if (Objects.isNull(detailMap) || !detailMap.containsKey(spotCheck.getId())) {
                log.info("【{}】点检方案明细不存在，请检查数据",spotCheck.getId());
                continue;
            }
            //检测零件
            if(spotCheck.getSpotCheckOrderType() == MesExtEnumUtil.SPOT_CHECK_ORDER_TYPE.FIRST_PRODUCT_INSPECTION.getValue() && (Objects.isNull(checkPartListMap) || !checkPartListMap.containsKey(spotCheck.getId()))){
                log.info("【{}】点检方案类型为【{}】配置零件号为空，请检查数据", spotCheck.getId(), MesExtEnumUtil.SPOT_CHECK_ORDER_TYPE.FIRST_PRODUCT_INSPECTION.getDescription());
                continue;
            }
            mesEquipmentSpotChecks.add(spotCheck);
        }
        return mesEquipmentSpotChecks;
    }

    private boolean checkTime(MesEquipmentSpotCheck spotCheck) {
        if(null == MesExtEnumUtil.EQUIPMENT_CHECK_INSPECTION_TRIGGER_CONFIG.valueOfDescription(spotCheck.getTimeTriggerMode())){
            log.info("时间触发方式{}不存在，请检查数据", spotCheck.getTimeTriggerMode());
            return true;
        }
        //触发窗口时间
        if(MesExtEnumUtil.EQUIPMENT_CHECK_INSPECTION_TRIGGER_CONFIG.OPTION1.getValue() == spotCheck.getTimeTriggerMode()){
            try {
                if(!DateUtilExt.between(DateUtil.SHORT_FORMAT_HOUR_MINUTE,TimeTool.getTimeShortWithColon(),TimeTool.pareDateToString(DateUtil.SHORT_FORMAT_HOUR_MINUTE,DateUtil.addMinutes(new Date(),30)),spotCheck.getTriggerWindowTime())){
                    log.info("{},触发窗口时间{}不产生单据，请检查数据", spotCheck.getSpotCheckCode(), spotCheck.getTriggerWindowTime());
                    return true;
                }
            } catch (Exception e) {
                log.info("{},触发窗口时间格式异常", spotCheck.getSpotCheckCode());
                return true;
            }
        }else{
            //查询执行成功的单据信息
            MesSpotCheckOrder finishedOrder = getMesSpotCheckOrder(spotCheck);
            if(!Objects.isNull(finishedOrder)){
                spotCheck.setLastSpotCheckTime(finishedOrder.getModifyDatetime());
                if(Integer.parseInt(DateUtil.minutesBetweenTwoTime(new Date(),TimeTool.stringParseToDate(finishedOrder.getModifyDatetime()))+"") < spotCheck.getTriggerWindowInterval()){
                    log.info("最后点检完成时间:{},距现在不足{}分钟，不创建单据",finishedOrder.getModifyDatetime(), spotCheck.getTriggerWindowInterval());
                    return true;
                }
            }
        }
        return false;
    }

    private MesSpotCheckOrder getMesSpotCheckOrder(MesEquipmentSpotCheck spotCheck) {
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(spotCheck.getOrganizeCode());
        DdlPreparedPack.getNumEqualPack(spotCheck.getId(),"spotCheckId", packBean);
        DdlPreparedPack.getNumEqualPack(MesExtEnumUtil.SPOT_CHECK_ORDER_STATUS.COMPLETE.getValue(), "status", packBean);
        DdlPreparedPack.getOrderBy("modifyDatetime",CommonEnumUtil.ASC_OR_DESC.DESC.getValue(),packBean);
        return baseRDao.getByProperty(packBean);
    }

    private void saveMesEquipmentSpotCheckPartList(String userName,Map<Long, List<MesEquipmentSpotCheckPart>> checkPartListMap, Long spotCheckId, Long spotCheckOrderId) {
        if (!Objects.isNull(checkPartListMap) && checkPartListMap.containsKey(spotCheckId)) {
            List<MesSpotCheckOrderPart> spotCheckOrderPartList = new ArrayList<>();
            checkPartListMap.get(spotCheckId).forEach(k -> {
                MesSpotCheckOrderPart spotCheckOrderPart = new MesSpotCheckOrderPart();
                spotCheckOrderPart.setSpotCheckOrderId(spotCheckOrderId);
                BeanUtils.copyProperties(k, spotCheckOrderPart, "id");
                spotCheckOrderPart.setSystemSyncStatus(CommonEnumUtil.FALSE);
                spotCheckOrderPart.setSystemSyncDatetime("");
                ConvertBean.serviceModelInitialize(spotCheckOrderPart, userName);
                spotCheckOrderPartList.add(spotCheckOrderPart);
            });
            spotCheckOrderPartRepository.saveAll(spotCheckOrderPartList);
        }
    }

    private void saveMesSpotCheckOrderDetails(String userName, Map<Long, List<MesEquipmentSpotCheckDetail>> detailMap, Long spotCheckId, MesSpotCheckOrder mesSpotCheckOrder) {
        List<MesSpotCheckOrderResult> spotCheckOrderResults = new ArrayList<>();
        detailMap.get(spotCheckId).forEach(k -> {
            MesSpotCheckOrderResult result = new MesSpotCheckOrderResult();
            BeanUtils.copyProperties(k, result, "id");
            result.setSpotCheckOrderNo(mesSpotCheckOrder.getSpotCheckOrderNo());
            result.setPid(mesSpotCheckOrder.getId());
            result.setWorkCenterCode(mesSpotCheckOrder.getWorkCenterCode());
            result.setWorkCellCode(mesSpotCheckOrder.getWorkCellCode());
            result.setSpotCheckItemId(k.getId());
            result.setTaskItemName(k.getSpotCheckItemName());
            result.setSpotCheckStandardValue(k.getStandardValue());
            result.setSystemSyncStatus(CommonEnumUtil.FALSE);
            result.setSystemSyncDatetime("");
            ConvertBean.serviceModelInitialize(result, userName);
            spotCheckOrderResults.add(result);
        });
        spotCheckOrderResultRepository.saveAll(spotCheckOrderResults);
    }

    private Map<Long, MesSpotCheckOrder> getMesSpotCheckOrderMap(String  organizeCode, List<Long> spotCheckIdList) {
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getInPackList(spotCheckIdList,"spotCheckId", packBean);
        DdlPreparedPack.getNumEqualPack(MesExtEnumUtil.SPOT_CHECK_ORDER_STATUS.CREATE.getValue(), "status", packBean);
        List<MesSpotCheckOrder> mesSpotCheckOrderList = baseRDao.findByHqlWhere(packBean);
        Map<Long, MesSpotCheckOrder> spotCheckOrderMap = null;
        if (!CollectionUtils.isEmpty(mesSpotCheckOrderList)) {
            spotCheckOrderMap = new HashMap<>();
            for (MesSpotCheckOrder mesSpotCheckOrder : mesSpotCheckOrderList) {
                spotCheckOrderMap.put(mesSpotCheckOrder.getSpotCheckId(),mesSpotCheckOrder);
            }
        }
        return spotCheckOrderMap;
    }

    @Override
    public MesSpotCheckOrder queryMesSpotCheckOrderResultPrint(MesSpotCheckOrder mesSpotCheckOrder) {
        //获取点检方案明细
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(mesSpotCheckOrder.getOrganizeCode());
        DdlPreparedPack.getNumEqualPack(mesSpotCheckOrder.getId(), "pid", packBean);
        List<MesSpotCheckOrderResult> detailList = spotCheckOrderResultRepository.findByHqlWhere(packBean);
        for (MesSpotCheckOrderResult mesSpotCheckOrderResult : detailList) {
            if (mesSpotCheckOrderResult.getIsTolerance()==1){
                mesSpotCheckOrderResult.setIsTolerance(Integer.parseInt(mesSpotCheckOrderResult.getSpotCheckSetValue())-Integer.parseInt(mesSpotCheckOrderResult.getSpotCheckRealValue()) );
            }else {
                mesSpotCheckOrderResult.setIsTolerance(0);
            }
        }
        mesSpotCheckOrder.setCheckOrderResults(detailList);
        MesRouteProcessCell routeProcessCell=new MesRouteProcessCell();
        routeProcessCell.setOrganizeCode(mesSpotCheckOrder.getOrganizeCode());
        routeProcessCell.setWorkCenterCode(mesSpotCheckOrder.getWorkCenterCode());
        routeProcessCell.setWorkCellCode(mesSpotCheckOrder.getWorkCellCode());
        Pager pager = new Pager();
        pager.setCurrentPage(0);
        pager.setPageSize(50);
        ListPager listPager = stateMachineRouteDesignService.queryStateMachineRouteProcessCellByPager(routeProcessCell, pager);
        List<MesRouteProcessCell> routeProcessCellList = listPager.getObjectList();
        if (!CollectionUtils.isEmpty(routeProcessCellList)){
            mesSpotCheckOrder.setProcessName(routeProcessCellList.get(0).getProcessCodeName());
        }
        mesSpotCheckOrder.setTemplateNo(mesConfigService.getCfgValue(mesSpotCheckOrder.getOrganizeCode(), MesCommonConstant.QC_ORDER_PRINT_TEMPLATE) );
        return mesSpotCheckOrder;
    }
}
